@@ -22,7 +22,7 @@ use File::Basename;            # to get the filename and directory path
 use File::HomeDir;
 use Log::Log4perl qw(get_logger :levels);
 use Exporter qw/import/;
-our @EXPORT_OK = qw/readSettings modify_line_breaks_settings get_indentation_settings_for_this_object get_every_or_custom_value get_indentation_information get_object_attribute_for_indentation_settings alignment_at_ampersand_settings %masterSettings/;
+our @EXPORT_OK = qw/yaml_read_settings yaml_modify_line_breaks_settings yaml_get_indentation_settings_for_this_object yaml_poly_switch_get_every_or_custom_value yaml_get_indentation_information yaml_get_object_attribute_for_indentation_settings yaml_alignment_at_ampersand_settings yaml_get_textwrap_removeparagraphline_breaks %masterSettings yaml_get_columns/;
 
 # Read in defaultSettings.YAML file
 our $defaultSettings;
@@ -44,7 +44,7 @@ our @alignAtAmpersandInformation = (   {name=>"lookForAlignDelims",yamlname=>"de
                                        {name=>"justification",default=>"left"},
                                         );
     
-sub readSettings{
+sub yaml_read_settings{
   my $self = shift;
   
   # read the default settings
@@ -85,24 +85,29 @@ sub readSettings{
 
   # messages for indentconfig.yaml and/or .indentconfig.yaml
   if ( -e $indentconfig and !$switches{onlyDefault}) {
-        $logger->info("Reading path information from $indentconfig");
-        # if both indentconfig.yaml and .indentconfig.yaml exist
-        if ( -e File::HomeDir->my_home . "/indentconfig.yaml" and  -e File::HomeDir->my_home . "/.indentconfig.yaml") {
-              $logger->info("$homeDir/.indentconfig.yaml has been found, but $indentconfig takes priority");
-        } elsif ( -e File::HomeDir->my_home . "/indentconfig.yaml" ) {
-              $logger->info("(Alternatively $homeDir/.indentconfig.yaml can be used)");
-        } elsif ( -e File::HomeDir->my_home . "/.indentconfig.yaml" ) {
-              $logger->info("(Alternatively $homeDir/indentconfig.yaml can be used)");
-        }
-        
         # read the absolute paths from indentconfig.yaml
         $userSettings = YAML::Tiny->read( "$indentconfig" );
 
-        # output the contents of indentconfig to the log file
-        $logger->info(Dump \%{$userSettings->[0]});
-        
         # update the absolute paths
-        @absPaths = @{$userSettings->[0]->{paths}};
+        if($userSettings and (ref($userSettings->[0]) eq 'HASH') and $userSettings->[0]->{paths}){
+            $logger->info("Reading path information from $indentconfig");
+            # if both indentconfig.yaml and .indentconfig.yaml exist
+            if ( -e File::HomeDir->my_home . "/indentconfig.yaml" and  -e File::HomeDir->my_home . "/.indentconfig.yaml") {
+                  $logger->info("$homeDir/.indentconfig.yaml has been found, but $indentconfig takes priority");
+            } elsif ( -e File::HomeDir->my_home . "/indentconfig.yaml" ) {
+                  $logger->info("(Alternatively $homeDir/.indentconfig.yaml can be used)");
+            } elsif ( -e File::HomeDir->my_home . "/.indentconfig.yaml" ) {
+                  $logger->info("(Alternatively $homeDir/indentconfig.yaml can be used)");
+            }
+            
+            # output the contents of indentconfig to the log file
+            $logger->info(Dump \%{$userSettings->[0]});
+        
+            @absPaths = @{$userSettings->[0]->{paths}};
+        } else {
+            $logger->warn("*The paths field cannot be read from $indentconfig; this means it is either empty or contains invalid YAML");
+            $logger->warn("See https://latexindentpl.readthedocs.io/en/latest/sec-indent-config-and-settings.html for an example");
+        }
   } else {
      if($switches{onlyDefault}) {
         $logger->info("*-d switch active: only default settings requested");
@@ -251,6 +256,23 @@ sub readSettings{
                                 }
                             }
                           }
+                      } elsif (ref($firstLevelValue) eq "ARRAY") {
+                            # update amalgamate in master settings
+                            if(ref(${$firstLevelValue}[0]) eq "HASH" and defined ${$firstLevelValue}[0]{amalgamate}){
+                               ${$masterSettings{$firstLevelKey}[0]}{amalgamate} = ${$firstLevelValue}[0]{amalgamate};
+                               shift @{$firstLevelValue} if ${$masterSettings{$firstLevelKey}[0]}{amalgamate};
+                            }
+
+                            # if amalgamate is set to 1, then append
+                            if(${$masterSettings{$firstLevelKey}[0]}{amalgamate}){
+                                # loop through the other settings
+                                foreach (@{$firstLevelValue}){
+                                    push (@{$masterSettings{$firstLevelKey}},$_);
+                                }
+                            } else {
+                                # otherwise overwrite
+                                $masterSettings{$firstLevelKey} = $firstLevelValue;
+                            }
                       } else {
                             $masterSettings{$firstLevelKey} = $firstLevelValue;
                       }
@@ -370,7 +392,7 @@ sub readSettings{
         # loop through each of the settings specified in the -y switch
         foreach(@yamlSettings){
             # split each value at semi-colon
-            my (@keysValues) = split(/(?<!\\):/,$_);
+            my (@keysValues) = split(/(?<!(?:\\|\[)):(?!\])/,$_);
 
             # $value will always be the last element
             my $value = $keysValues[-1];
@@ -431,7 +453,7 @@ sub readSettings{
   return;
 }
 
-sub get_indentation_settings_for_this_object{
+sub yaml_get_indentation_settings_for_this_object{
     my $self = shift;
 
     # create a name for previously found settings
@@ -449,13 +471,13 @@ sub get_indentation_settings_for_this_object{
 
         # check for noAdditionalIndent and indentRules
         # otherwise use defaultIndent
-        my $indentation = $self->get_indentation_information;
+        my $indentation = $self->yaml_get_indentation_information;
 
         # check for alignment at ampersand settings
-        $self->alignment_at_ampersand_settings;
+        $self->yaml_alignment_at_ampersand_settings;
 
         # check for line break settings
-        $self->modify_line_breaks_settings;
+        $self->yaml_modify_line_breaks_settings if $is_m_switch_active;
 
         # store the settings
         %{${previouslyFoundSettings}{$storageName}} = (
@@ -465,6 +487,8 @@ sub get_indentation_settings_for_this_object{
                         EndStartsOnOwnLine=>${$self}{EndStartsOnOwnLine},
                         EndFinishesWithLineBreak=>${$self}{EndFinishesWithLineBreak},
                         removeParagraphLineBreaks=>${$self}{removeParagraphLineBreaks},
+                        textWrapOptions=>${$self}{textWrapOptions},
+                        columns=>${$self}{columns},
                       );
 
         # don't forget alignment settings!
@@ -492,7 +516,7 @@ sub get_indentation_settings_for_this_object{
     return;
 }
 
-sub alignment_at_ampersand_settings{
+sub yaml_alignment_at_ampersand_settings{
     my $self = shift;
 
     # if the YamlName is, for example, optionalArguments, mandatoryArguments, heading, then we'll be looking for information about the *parent*
@@ -534,17 +558,15 @@ sub alignment_at_ampersand_settings{
     return;
 }
 
-sub modify_line_breaks_settings{
-    # return with undefined values unless the -m switch is active
-    return unless $is_m_switch_active;
-
+sub yaml_modify_line_breaks_settings{
     my $self = shift;
 
     # grab the logging object
     my $logger = get_logger("Document");
 
     # details to the log file
-    $logger->trace("*-m modifylinebreaks switch active, looking for settings for ${$self}{name} ") if $is_t_switch_active;
+    $logger->trace("*-m modifylinebreaks switch active") if $is_t_switch_active;
+    $logger->trace("looking for polyswitch, textWrapOptions, removeParagraphLineBreaks, oneSentencePerLine settings for ${$self}{name} ") if $is_t_switch_active;
 
     # some objects, e.g ifElseFi, can have extra assignments, e.g ElseStartsOnOwnLine
     my @toBeAssignedTo = ${$self}{additionalAssignments} ? @{${$self}{additionalAssignments}} : ();
@@ -554,47 +576,201 @@ sub modify_line_breaks_settings{
 
     # we can efficiently loop through the following
     foreach (@toBeAssignedTo){
-                    $self->get_every_or_custom_value(
+                    $self->yaml_poly_switch_get_every_or_custom_value(
                                     toBeAssignedTo=>$_,
                                     toBeAssignedToAlias=> ${$self}{aliases}{$_} ?  ${$self}{aliases}{$_} : $_,
                                   );
-      }
+      };
 
-    # paragraph line break settings
-    ${$self}{removeParagraphLineBreaks} = ${$masterSettings{modifyLineBreaks}{removeParagraphLineBreaks}}{all};
+    $self->yaml_get_textwrap_removeparagraphline_breaks;
+    return;
+}
 
-    return if(${$self}{removeParagraphLineBreaks});
+sub yaml_get_textwrap_removeparagraphline_breaks{
+    my $self = shift;
+    
+    # grab the logging object
+    my $logger = get_logger("Document");
 
-    # the removeParagraphLineBreaks can contain fields that are hashes or scalar, for example:
-    # 
-    # removeParagraphLineBreaks:
-    #     all: 0
-    #     environments: 0
-    # or
-    # removeParagraphLineBreaks:
-    #     all: 0
-    #     environments: 
-    #         quotation: 0
+    # textWrap and removeParagraphLineBreaks settings
+    foreach ("textWrapOptions","removeParagraphLineBreaks"){
+
+        # first check for either
+        #
+        # textWrapOptions:
+        #     all: 0
+        #
+        # or
+        #
+        # removeParagraphLineBreaks:
+        #     all: 0
+        #
+        # *IMPORTANT*
+        # even if all is set to 1, then it can still be disabled on either a
+        # 
+        # per-object:
+        #   
+        #   for example
+        #
+        #       textWrapOptions:
+        #           all:
+        #               except:
+        #                   - environments
+        #
+        #   will disable textWrapOptions for *all* environments
+        #
+        # per-name
+        # 
+        #   for example
+        #
+        #       textWrapOptions:
+        #           all: 
+        #               except:
+        #                   - itemize
+        #
+        #   will disable textWrapOptions for itemize
+        
+        # if 'all' is set as a hash, then the default value is 1, to be turned  off (possibly) later
+        ${$self}{$_} = ( ref ${$masterSettings{modifyLineBreaks}{$_}}{all} eq "HASH" ? 1 : ${$masterSettings{modifyLineBreaks}{$_}}{all});
+
+        # get the columns
+        if($_ eq "textWrapOptions" and ${$masterSettings{modifyLineBreaks}{textWrapOptions}}{perCodeBlockBasis}){
+            $self->yaml_get_columns;
+        }
+        
+        # name of the object in the modifyLineBreaks yaml (e.g environments, ifElseFi, etc)
+        my $YamlName = ${$self}{modifyLineBreaksYamlName};
+        
+        # if the YamlName is either optionalArguments or mandatoryArguments, then we'll be looking for information about the *parent*
+        my $name = ($YamlName =~ m/Arguments/) ? ${$self}{parent} : ${$self}{name};
+
+        # move to the next <thing> if
+        #
+        #   textWrapOptions/removeParagraphLineBreaks::
+        #       all: 1
+        #
+        if(${$self}{$_} 
+                    and 
+                ref ${$masterSettings{modifyLineBreaks}{$_}}{all} ne "HASH" 
+                    and 
+                ${$masterSettings{modifyLineBreaks}{$_}}{all}){
+               $logger->trace("$_ for $name is ${$self}{$_}") if $is_t_switch_active;
+               next;  
+        };
+
+        # otherwise, look for exceptions, either through
+        #
+        #   textWrapOptions/removeParagraphLineBreaks:
+        #       all:
+        #           except:
+        #               - <*type* of thing or *name* of thing>
+        #
+        # so, for example, the following (per code-block) is acceptable
+        # which makes an exception for all *environments*
+        #
+        #       all:
+        #           except:
+        #               - 'environments'
+        #
+        # the following (per-name) is acceptable 
+        # which only makes an exception for things called itemize
+        #
+        #       all:
+        #           except:
+        #               - 'itemize'
+        #
+        if(${$self}{$_} 
+                and 
+           defined ${${$masterSettings{modifyLineBreaks}{$_}}{all}}{except} 
+                and 
+           ref ${${$masterSettings{modifyLineBreaks}{$_}}{all}}{except} eq "ARRAY"
+         ){
+              my %except = map { $_ => 1 } @{${${$masterSettings{modifyLineBreaks}}{$_}}{all}{except}};
+              if( $except{$name} or $except{$YamlName}){
+                ${$self}{$_} = 0;
+                my $detail = ($except{$name} ? "per-name" : "per-code-block-type");
+                $logger->trace("$_ for $name is ${$self}{$_} (found as exception $detail, see $_:all:except)") if $is_t_switch_active;
+                next;
+              }
+        } else {
+            # or otherwise through, for example
+            #
+            #   all: 0
+            #   ifElseFi: 1
+            #
+            # the textWrapOptions/removeParagraphLineBreaks can contain fields that are hashes or scalar
+            # 
+            if(ref ${$masterSettings{modifyLineBreaks}{$_}}{$YamlName} eq "HASH"){
+                # textWrapOptions/removeParagraphLineBreaks:
+                #     all: 0
+                #     environments: 
+                #         quotation: 0
+                $logger->trace("*$YamlName specified with fields in $_, looking for $name") if $is_t_switch_active;
+                ${$self}{$_} = ${${$masterSettings{modifyLineBreaks}{$_}}{$YamlName}}{$name} if (defined ${${$masterSettings{modifyLineBreaks}{$_}}{$YamlName}}{$name});
+            } elsif(defined ${$masterSettings{modifyLineBreaks}{$_}}{$YamlName}){
+                # textWrapOptions/removeParagraphLineBreaks:
+                #     all: 0
+                #     environments: 0
+                $logger->trace("*$YamlName specified with just a number in $_ ${$masterSettings{modifyLineBreaks}{$_}}{$YamlName}") if $is_t_switch_active;
+                ${$self}{$_} = ${$masterSettings{modifyLineBreaks}{$_}}{$YamlName} if (defined ${$masterSettings{modifyLineBreaks}{$_}}{$YamlName});
+            }
+        }
+
+        # summary to log file
+        $logger->trace("$_ for $name is ${$self}{$_}") if $is_t_switch_active;
+    }
+
+    return;
+}
+
+sub yaml_get_columns{
+    my $self = shift;
 
-    # name of the object in the modifyLineBreaks yaml (e.g environments, ifElseFi, etc)
     my $YamlName = ${$self}{modifyLineBreaksYamlName};
 
-    # if the YamlName is either optionalArguments or mandatoryArguments, then we'll be looking for information about the *parent*
-    my $name = ($YamlName =~ m/Arguments/) ? ${$self}{parent} : ${$self}{name};
-
-    if(ref ${$masterSettings{modifyLineBreaks}{removeParagraphLineBreaks}}{$YamlName} eq "HASH"){
-        $logger->trace("*$YamlName specified with fields in removeParagraphLineBreaks, looking for $name") if $is_t_switch_active;
-        ${$self}{removeParagraphLineBreaks} = ${${$masterSettings{modifyLineBreaks}{removeParagraphLineBreaks}}{$YamlName}}{$name}||0;
-    } else {
-        if(defined ${$masterSettings{modifyLineBreaks}{removeParagraphLineBreaks}}{$YamlName}){
-            $logger->trace("*$YamlName specified with just a number in removeParagraphLineBreaks ${$masterSettings{modifyLineBreaks}{removeParagraphLineBreaks}}{$YamlName}") if $is_t_switch_active;
-            ${$self}{removeParagraphLineBreaks} = ${$masterSettings{modifyLineBreaks}{removeParagraphLineBreaks}}{$YamlName};
+    # the columns settings can have a variety of different ways of being specified
+    if(ref ${$masterSettings{modifyLineBreaks}{textWrapOptions}}{columns} eq "HASH"){
+        # assign default value of $columns
+        my $columns;
+        if(defined ${${$masterSettings{modifyLineBreaks}{textWrapOptions}}{columns}}{default}){
+            $columns = ${${$masterSettings{modifyLineBreaks}{textWrapOptions}}{columns}}{default};
+        } else {
+            $columns = 80;
         }
+
+        # possibly specify object wrapping on a per-name basis
+        if(ref ${${$masterSettings{modifyLineBreaks}{textWrapOptions}}{columns}}{$YamlName} eq "HASH"){
+            # for example:
+            #   modifyLineBreaks:
+            #       textWrapOptions:
+            #           columns: 
+            #               default: 80
+            #               environments:
+            #                   default: 80
+            #                   something: 10
+            #                   another: 20
+            if(defined ${${${$masterSettings{modifyLineBreaks}{textWrapOptions}}{columns}}{$YamlName}}{${$self}{name}}){
+                $columns = ${${${$masterSettings{modifyLineBreaks}{textWrapOptions}}{columns}}{$YamlName}}{${$self}{name}};
+            } elsif (${${${$masterSettings{modifyLineBreaks}{textWrapOptions}}{columns}}{$YamlName}}{default}){
+                $columns = ${${${$masterSettings{modifyLineBreaks}{textWrapOptions}}{columns}}{$YamlName}}{default};
+            }
+        } else {
+            # for example:
+            #   modifyLineBreaks:
+            #       textWrapOptions:
+            #           columns: 
+            #               default: 80
+            #               environments: 10
+            $columns = ${${$masterSettings{modifyLineBreaks}{textWrapOptions}}{columns}}{$YamlName};
+        }
+        ${$self}{columns} = $columns;
+    } else {
+        ${$self}{columns} = ${$masterSettings{modifyLineBreaks}{textWrapOptions}}{columns};
     }
     return;
 }
 
-sub get_every_or_custom_value{
+sub yaml_poly_switch_get_every_or_custom_value{
   my $self = shift;
   my %input = @_;
 
@@ -621,19 +797,19 @@ sub get_every_or_custom_value{
 
   # check for the *custom* value
   if (defined $customValue){
-      $logger->trace("$name: $toBeAssignedToAlias=$customValue, (*custom* value) adjusting $toBeAssignedTo") if($is_t_switch_active);
+      $logger->trace("$name: $toBeAssignedToAlias=$customValue, (*per-name* value) adjusting $toBeAssignedTo") if($is_t_switch_active);
       ${$self}{$toBeAssignedTo} = $customValue !=0 ? $customValue : undef;
    } else {
       # check for the *every* value
       if (defined $everyValue and $everyValue != 0){
-          $logger->trace("$name: $toBeAssignedToAlias=$everyValue, (*every* value) adjusting $toBeAssignedTo") if($is_t_switch_active);
+          $logger->trace("$name: $toBeAssignedToAlias=$everyValue, (*global* value) adjusting $toBeAssignedTo") if($is_t_switch_active);
           ${$self}{$toBeAssignedTo} = $everyValue;
       }
    }
   return;
 }
 
-sub get_indentation_information{
+sub yaml_get_indentation_information{
     my $self = shift;
 
     #**************************************
@@ -681,7 +857,7 @@ sub get_indentation_information{
     my $name = (defined ${$self}{nameForIndentationSettings}) ? ${$self}{nameForIndentationSettings} : ${$self}{name};
 
     # if the YamlName is not optionalArguments, mandatoryArguments, heading (possibly others) then assume we're looking for 'body'
-    my $YamlName = $self->get_object_attribute_for_indentation_settings;
+    my $YamlName = $self->yaml_get_object_attribute_for_indentation_settings;
 
     # grab the logging object
     my $logger = get_logger("Document");
@@ -735,7 +911,7 @@ sub get_indentation_information{
     return $masterSettings{defaultIndent};
 }
 
-sub get_object_attribute_for_indentation_settings{
+sub yaml_get_object_attribute_for_indentation_settings{
     # when looking for noAdditionalIndent or indentRules, we may need to determine
     # which thing we're looking for, e.g
     #
