@@ -18,7 +18,10 @@ use strict;
 use warnings;
 use LatexIndent::Tokens qw/%tokens/;
 use LatexIndent::TrailingComments qw/$trailingCommentRegExp/;
-use LatexIndent::Switches qw/$is_t_switch_active $is_tt_switch_active/;
+use LatexIndent::Switches qw/$is_t_switch_active $is_tt_switch_active $is_m_switch_active /;
+use LatexIndent::LogFile qw/$logger/;
+use LatexIndent::IfElseFi qw/$ifElseFiBasicRegExp/;
+use LatexIndent::Special qw/$specialBeginBasicRegExp/;
 use Exporter qw/import/;
 our @ISA = "LatexIndent::Document"; # class inheritance, Programming Perl, pg 321
 our @EXPORT_OK = qw/find_mandatory_arguments get_mand_arg_reg_exp/;
@@ -32,8 +35,8 @@ sub find_mandatory_arguments{
     # pick out the mandatory arguments
     while(${$self}{body} =~ m/$mandArgRegExp\h*($trailingCommentRegExp)*(.*)/s){
         # log file output
-        $self->logger("Mandatory argument found, body in ${$self}{name}",'heading') if $is_t_switch_active;
-        $self->logger("(last argument)") if($9 eq '' and $is_t_switch_active);
+        $logger->trace("*Mandatory argument found, body in ${$self}{name}") if $is_t_switch_active;
+        $logger->trace("(last argument)") if($9 eq '' and $is_t_switch_active);
 
         ${$self}{body} =~ s/
                             $mandArgRegExp(\h*)($trailingCommentRegExp)*(.*)
@@ -110,7 +113,7 @@ sub get_mand_arg_reg_exp{
     return $mandArgRegExp;
 }
 
-sub get_object_attribute_for_indentation_settings{
+sub yaml_get_object_attribute_for_indentation_settings{
     my $self = shift;
     
     return ${$self}{modifyLineBreaksYamlName};
@@ -120,11 +123,13 @@ sub tasks_particular_to_each_object{
     my $self = shift;
 
     # search for ifElseFi blocks
-    $self->find_ifelsefi;
+    $self->find_ifelsefi if ${$self}{body} =~ m/$ifElseFiBasicRegExp/s;
 
     # search for special begin/end
-    $self->find_special;
+    $self->find_special if ${$self}{body} =~ m/$specialBeginBasicRegExp/s;
 
+    # comma poly-switch check
+    $self->comma_else if $is_m_switch_active;
 }
 
 1;
