@@ -1,5 +1,5 @@
 #!/usr/bin/env perl
-#   latexindent.pl, version 3.4.2, 2018-04-27
+#   latexindent.pl, version 3.7.1, 2019-09-07
 #
 #	This program is free software: you can redistribute it and/or modify
 #	it under the terms of the GNU General Public License as published by
@@ -44,6 +44,9 @@ GetOptions (
     "help|h"=>\$switches{showhelp},
     "cruft|c=s"=>\$switches{cruftDirectory},
     "screenlog|sl"=>\$switches{screenlog},
+    "replacement|r"=>\$switches{replacement},
+    "onlyreplacement|rr"=>\$switches{onlyreplacement},
+    "replacementrespectverb|rv"=>\$switches{replacementRespectVerb},
 );
 
 # check local settings doesn't interfer with reading the file;
@@ -73,6 +76,6 @@ if(defined($switches{readLocalSettings}) and ($switches{readLocalSettings} eq ''
 # allow STDIN as input, if a filename is not present
 unshift( @ARGV, '-' ) unless @ARGV;
 
-my $document = bless ({name=>"masterDocument",fileName=>$ARGV[0],switches=>\%switches},"LatexIndent::Document");
+my $document = bless ({name=>"masterDocument",modifyLineBreaksYamlName=>"masterDocument",fileName=>$ARGV[0],switches=>\%switches},"LatexIndent::Document");
 $document->latexindent;
 exit(0);
