@@ -1,7 +1,7 @@
 #!/usr/bin/env perl
-# $Id: tlmgrgui.pl 44719 2017-06-28 23:40:26Z preining $
+# $Id: tlmgrgui.pl 50334 2019-03-11 01:56:18Z preining $
 #
-# Copyright 2009-2017 Norbert Preining
+# Copyright 2009-2019 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
 #
@@ -13,8 +13,8 @@
 $^W = 1;
 use strict;
 
-my $guisvnrev = '$Revision: 44719 $';
-my $guidatrev = '$Date: 2017-06-29 01:40:26 +0200 (Thu, 29 Jun 2017) $';
+my $guisvnrev = '$Revision: 50334 $';
+my $guidatrev = '$Date: 2019-03-11 02:56:18 +0100 (Mon, 11 Mar 2019) $';
 my $tlmgrguirevision;
 if ($guisvnrev =~ m/: ([0-9]+) /) {
   $tlmgrguirevision = $1;
@@ -1367,7 +1367,7 @@ sub do_paper_settings {
       if (($p eq "context") && !defined($localtlpdb->get_package("bin-context"))) {
         next;
       }
-      $l{$p} = $back_config_pap->Label(-text => __("Default paper for") . " $p", -anchor => "w");
+      $l{$p} = $back_config_pap->Label(-text => __("Default paper for %s", $p), -anchor => "w");
       $m{$p} = $back_config_pap->Label(-textvariable => \$changedpaper{$p}, -anchor => "w");
       $settings_label{$p} = $m{$p};
       $r{$p} = $back_config_pap->Button(-text => __("Change"),
@@ -1768,11 +1768,11 @@ sub change_paper {
 sub select_paper {
   my $back_config = shift;
   my $prog = shift;
-  my $foo = $back_config->Toplevel(-title => __("Select paper format for") . " $prog");
+  my $foo = $back_config->Toplevel(-title => __("Select paper format for %s", $prog));
   $foo->transient($back_config);
   $foo->grab();
   my $var = $changedpaper{$prog};
-  my $opt = $foo->BrowseEntry(-label => __("Default paper for") . " $prog", -variable => \$var);
+  my $opt = $foo->BrowseEntry(-label => __("Default paper for %s", $prog), -variable => \$var);
   foreach my $p (sort @{$papers{$prog}}) {
     $opt->insert("end",$p);
   }
@@ -2489,7 +2489,7 @@ sub cb_edit_string_or_dir {
   my $sw = $mw->Toplevel(-title => __("Edit directory"));
   $sw->transient($mw);
   $sw->withdraw;
-  $sw->Label(-text => __("New value for") . " $what:")->pack(@p_ii);
+  $sw->Label(-text => __("New value for %s:", $what))->pack(@p_ii);
   my $entry = $sw->Entry(-text => $cur, -width => 30);
   $entry->pack(@p_ii);
   $sw->Button(-text => __("Choose Directory"),
@@ -2538,13 +2538,13 @@ sub cb_edit_location {
       sub {
         if ($val !~ m/^  /) {
           $val = "";
-          $okbutton->configure(-state => 'disabled');
+          # $okbutton->configure(-state => 'disabled');
         } elsif ($val =~ m!(http|ftp)://!) {
           $val = TeXLive::TLUtils::extract_mirror_entry($val);
-          $okbutton->configure(-state => 'normal');
+          # $okbutton->configure(-state => 'normal');
         } else {
           $val =~ s/^\s*//;
-          $okbutton->configure(-state => 'normal');
+          # $okbutton->configure(-state => 'normal');
         }
       },
     -variable => \$val);
@@ -2556,17 +2556,17 @@ sub cb_edit_location {
                       my $var = $sw->chooseDirectory();
                       if (defined($var)) {
                         $val = $var;
-                        $okbutton->configure(-state => 'normal');
+                        # $okbutton->configure(-state => 'normal');
                       }
                     })->pack(@left, @p_ii);
   $f1->Button(-text => __("Use standard net repository"),
     -command => sub {
                       $val = $TeXLiveURL;
-                      $okbutton->configure(-state => 'normal');
+                      # $okbutton->configure(-state => 'normal');
                     })->pack(@left, @p_ii);
   $f1->pack;
   my $f = $sw->Frame;
-  $okbutton = $f->Button(-text => __("Load"), -state => "disabled",
+  $okbutton = $f->Button(-text => __("Load"), # -state => "disabled",
     -command => sub { 
                       if ($val) {
                         $location = $val;
