@@ -68,166 +68,150 @@ end
 nodes                       = nodes or { }
 local nodes                 = nodes
 
-local nodecodes             = nodes.nodecodes
+local nodecodes               = nodes.nodecodes
 
-nodes.tostring              = node.tostring or tostring
-nodes.copy                  = node.copy
-nodes.copy_node             = node.copy
-nodes.copy_list             = node.copy_list
-nodes.delete                = node.delete
-nodes.dimensions            = node.dimensions
-nodes.rangedimensions       = node.rangedimensions
-nodes.end_of_math           = node.end_of_math
-nodes.flush                 = node.flush_node
-nodes.flush_node            = node.flush_node
-nodes.flush_list            = node.flush_list
-nodes.free                  = node.free
-nodes.insert_after          = node.insert_after
-nodes.insert_before         = node.insert_before
-nodes.hpack                 = node.hpack
-nodes.new                   = node.new
-nodes.tail                  = node.tail
-nodes.traverse              = node.traverse
-nodes.traverse_id           = node.traverse_id
-nodes.traverse_char         = node.traverse_char
-nodes.slide                 = node.slide
-nodes.vpack                 = node.vpack
-nodes.fields                = node.fields
-nodes.is_node               = node.is_node
-nodes.setglue               = node.setglue
+nodes.tostring                = node.tostring or tostring
+nodes.copy                    = node.copy
+nodes.copy_node               = node.copy
+nodes.copy_list               = node.copy_list
+nodes.delete                  = node.delete
+nodes.dimensions              = node.dimensions
+nodes.rangedimensions         = node.rangedimensions
+nodes.end_of_math             = node.end_of_math
+nodes.flush                   = node.flush_node
+nodes.flush_node              = node.flush_node
+nodes.flush_list              = node.flush_list
+nodes.free                    = node.free
+nodes.insert_after            = node.insert_after
+nodes.insert_before           = node.insert_before
+nodes.hpack                   = node.hpack
+nodes.new                     = node.new
+nodes.tail                    = node.tail
+nodes.traverse                = node.traverse
+nodes.traverse_id             = node.traverse_id
+nodes.traverse_char           = node.traverse_char
+nodes.traverse_glyph          = node.traverse_glyph
+nodes.traverse_list           = node.traverse_list
+nodes.slide                   = node.slide
+nodes.vpack                   = node.vpack
+nodes.fields                  = node.fields
+nodes.is_node                 = node.is_node
+nodes.setglue                 = node.setglue
+nodes.uses_font               = node.uses_font
 
-nodes.first_glyph           = node.first_glyph
-nodes.has_glyph             = node.has_glyph or node.first_glyph
+nodes.first_glyph             = node.first_glyph
+nodes.has_glyph               = node.has_glyph or node.first_glyph
 
-nodes.current_attr          = node.current_attr
-nodes.has_field             = node.has_field
-nodes.last_node             = node.last_node
-nodes.usedlist              = node.usedlist
-nodes.protrusion_skippable  = node.protrusion_skippable
-nodes.check_discretionaries = node.check_discretionaries
-nodes.write                 = node.write
+nodes.current_attr            = node.current_attr
+nodes.has_field               = node.has_field
+nodes.last_node               = node.last_node
+nodes.usedlist                = node.usedlist
+nodes.protrusion_skippable    = node.protrusion_skippable
+nodes.check_discretionaries   = node.check_discretionaries
+nodes.write                   = node.write
+nodes.flatten_discretionaries = node.flatten_discretionaries
 
-nodes.count                 = node.count
-nodes.length                = node.length
+nodes.count                   = node.count
+nodes.length                  = node.length
 
-nodes.has_attribute         = node.has_attribute
-nodes.set_attribute         = node.set_attribute
-nodes.find_attribute        = node.find_attribute
-nodes.unset_attribute       = node.unset_attribute
+nodes.has_attribute           = node.has_attribute
+nodes.set_attribute           = node.set_attribute
+nodes.find_attribute          = node.find_attribute
+nodes.unset_attribute         = node.unset_attribute
 
-nodes.protect_glyph         = node.protect_glyph
-nodes.protect_glyphs        = node.protect_glyphs
-nodes.unprotect_glyph       = node.unprotect_glyph
-nodes.unprotect_glyphs      = node.unprotect_glyphs
-nodes.kerning               = node.kerning
-nodes.ligaturing            = node.ligaturing
-nodes.mlist_to_hlist        = node.mlist_to_hlist
+nodes.protect_glyph           = node.protect_glyph
+nodes.protect_glyphs          = node.protect_glyphs
+nodes.unprotect_glyph         = node.unprotect_glyph
+nodes.unprotect_glyphs        = node.unprotect_glyphs
+nodes.kerning                 = node.kerning
+nodes.ligaturing              = node.ligaturing
+nodes.hyphenating             = node.hyphenating
+nodes.mlist_to_hlist          = node.mlist_to_hlist
 
-nodes.effective_glue       = node.effective_glue
-nodes.getglue              = node.getglue
-nodes.setglue              = node.setglue
-nodes.is_zero_glue         = node.is_zero_glue
+nodes.effective_glue          = node.effective_glue
+nodes.getglue                 = node.getglue
+nodes.setglue                 = node.setglue
+nodes.is_zero_glue            = node.is_zero_glue
 
 nodes.tonode = function(n) return n end
 nodes.tonut  = function(n) return n end
 
-local n_getid           = node.getid
-local n_getlist         = node.getlist
-local n_getnext         = node.getnext
-local n_getprev         = node.getprev
-local n_getchar         = node.getchar
-local n_getfont         = node.getfont
-local n_getsubtype      = node.getsubtype
-local n_getfield        = node.getfield
-local n_getattr         = node.get_attribute
-local n_getdisc         = node.getdisc
-local n_getleader       = node.getleader
+-- These are never used in \CONTEXT, only as a gimmick in node operators
+-- so we keep them around.
 
-local n_setfield        = node.setfield
-local n_setattr         = n_setfield
+local n_getfield = node.getfield
+local n_getattr  = node.get_attribute
 
-local n_setnext         = node.setnext or -- always
-    function(c,n)
-        n_setfield(c,"next",n)
-    end
-local n_setprev         = node.setprev or -- always
-    function(c,p)
-        n_setfield(c,"prev",p)
-    end
-local n_setlist         = node.setlist or -- always
-    function(c,l)
-        n_setfield(c,"list",l)
-    end
-local n_setlink         = node.setlink or -- always
-    function(...)
-        -- not that fast but not used often anyway
-        local h = nil
-        for i=1,select("#",...) do
-            local n = (select(i,...))
-            if not n then
-                -- go on
-            elseif h then
-                n_setfield(h,"next",n)
-                n_setfield(n,"prev",h)
-            else
-                h = n
-            end
+local n_setfield = node.setfield
+local n_setattr  = n_setfield
+
+nodes.getfield = n_getfield
+nodes.setfield = n_setfield
+nodes.getattr  = n_getattr
+nodes.setattr  = n_setattr
+nodes.takeattr = nodes.unset_attribute
+
+local function n_getid     (n) return n_getfield(n,"id")      end
+local function n_getsubtype(n) return n_getfield(n,"subtype") end
+
+nodes.getid      = n_getid
+nodes.getsubtype = n_getsubtype
+
+local function n_getchar(n)   return n_getfield(n,"char")    end
+local function n_setchar(n,c) return n_setfield(n,"char",c)  end
+local function n_getfont(n)   return n_getfield(n,"font")    end
+local function n_setfont(n,f) return n_setfield(n,"font",f)  end
+
+nodes.getchar = n_getchar
+nodes.setchar = n_setchar
+nodes.getfont = n_getfont
+nodes.setfont = n_setfont
+
+local function n_getlist  (n)   return n_getfield(n,"list")     end
+local function n_setlist  (n,l) return n_setfield(n,"list",l)   end
+local function n_getleader(n)   return n_getfield(n,"leader")   end
+local function n_setleader(n,l) return n_setfield(n,"leader",l) end
+
+nodes.getlist   = n_getlist
+nodes.setlist   = n_setlist
+nodes.getleader = n_getleader
+nodes.setleader = n_setleader
+
+local function n_getnext(n)       return n_getfield(n,"next")    end
+local function n_setnext(n,nn)    return n_setfield(n,"next",nn) end
+local function n_getprev(n)       return n_getfield(n,"prev")    end
+local function n_setprev(n,pp)    return n_setfield(n,"prev",pp) end
+local function n_getboth(n)       return n_getfield(n,"prev"),    n_getfield(n,"next")    end
+local function n_setboth(n,pp,nn) return n_setfield(n,"prev",pp), n_setfield(n,"next",nn) end
+
+nodes.getnext = n_getnext
+nodes.setnext = n_setnext
+nodes.getprev = n_getprev
+nodes.setprev = n_setprev
+nodes.getboth = n_getboth
+nodes.setboth = n_setboth
+
+local function n_setlink(...)
+    -- not that fast but not used often anyway
+    local h = nil
+    for i=1,select("#",...) do
+        local n = select(i,...)
+        if not n then
+            -- go on
+        elseif h then
+            n_setfield(h,"next",n)
+            n_setfield(n,"prev",h)
+        else
+            h = n
         end
-        return h
-    end
-local n_setboth         = node.setboth or -- always
-    function(c,p,n)
-        n_setfield(c,"prev",p)
-        n_setfield(c,"next",n)
-    end
-
-nodes.setnext         = n_setnext
-nodes.setprev         = n_setprev
-nodes.setlink         = n_setlink
-nodes.setboth         = n_setboth
-nodes.setlist         = n_setlist
-
-nodes.getfield        = n_getfield
-nodes.setfield        = n_setfield
-nodes.getattr         = n_getattr
-nodes.setattr         = n_setattr
-nodes.takeattr        = nodes.unset_attribute
-
-nodes.getnext         = n_getnext
-nodes.getprev         = n_getprev
-nodes.getid           = n_getid
-nodes.getchar         = n_getchar
-nodes.getfont         = n_getfont
-nodes.getsubtype      = n_getsubtype
-nodes.getlist         = n_getlist
-nodes.getleader       = n_getleader
-nodes.getdisc         = n_getdisc
-
-if not node.getwhd then
-    function node.getwhd(n)
-        return n_getfield(n,"width"), n_getfield(n,"height"), n_getfield(n,"depth")
     end
+    return h
 end
 
-if not node.setwhd then
-    function node.setwhd(n,w,h,d)
-        n_setfield(n,"width",w or 0)
-        n_setfield(n,"height",h or 0)
-        n_setfield(n,"depth",d or 0)
-    end
-end
+nodes.setlink = n_setlink
 
-nodes.getwhd = node.getwhd
-nodes.setwhd = node.setwhd
-
-nodes.is_char         = node.is_char
-nodes.ischar          = node.is_char
-
-nodes.is_glyph        = node.is_glyph
-nodes.isglyph         = node.is_glyph
-
-nodes.getbox          = node.getbox or tex.getbox
-nodes.setbox          = node.setbox or tex.setbox
+nodes.getbox  = node.getbox or tex.getbox
+nodes.setbox  = node.setbox or tex.setbox
 
 local n_flush_node    = nodes.flush
 local n_copy_node     = nodes.copy
@@ -621,7 +605,7 @@ local messyhack    = table.tohash { -- temporary solution
     nodecodes.action,
 }
 
-table.setmetatableindex(keys,function(t,k)
+setmetatableindex(keys,function(t,k)
     local v = (k == "attributelist" or k == nodecodes.attributelist) and { } or getfields(k)
     if messyhack[k] then
         for i=1,#v do
@@ -638,7 +622,7 @@ table.setmetatableindex(keys,function(t,k)
     return v
 end)
 
-table.setmetatableindex(whatsitkeys,function(t,k)
+setmetatableindex(whatsitkeys,function(t,k)
     local v = getfields(whatsit_code,k)
     if v[ 0] then v[#v+1] = "next" v[ 0] = nil end
     if v[-1] then v[#v+1] = "prev" v[-1] = nil end
@@ -659,46 +643,3 @@ end
 
 nodes.keys   = keys       -- [id][subtype]
 nodes.fields = nodefields -- (n)
-
--- for the moment (pre 6380)
-
-if not nodes.unprotect_glyph then
-
-    local protect_glyph    = nodes.protect_glyph
-    local protect_glyphs   = nodes.protect_glyphs
-    local unprotect_glyph  = nodes.unprotect_glyph
-    local unprotect_glyphs = nodes.unprotect_glyphs
-
-    local getnext          = nodes.getnext
-    local setnext          = nodes.setnext
-
-    function nodes.protectglyphs(first,last)
-        if first == last then
-            return protect_glyph(first)
-        elseif last then
-            local nxt = getnext(last)
-            setnext(last)
-            local f, b = protect_glyphs(first)
-            setnext(last,nxt)
-            return f, b
-        else
-            return protect_glyphs(first)
-        end
-    end
-
-    function nodes.unprotectglyphs(first,last)
-        if first == last then
-            return unprotect_glyph(first)
-        elseif last then
-            local nxt = getnext(last)
-            setnext(last)
-            local f, b = unprotect_glyphs(first)
-            setnext(last,nxt)
-            return f, b
-        else
-            return unprotect_glyphs(first)
-        end
-    end
-
-end
-
