@@ -4,16 +4,16 @@ if not modules then modules = { } end modules ['typo-duc'] = {
     author    = "Hans Hagen, PRAGMA-ADE, Hasselt NL",
     copyright = "PRAGMA ADE / ConTeXt Development Team",
     license   = "see context related readme files",
-    comment   = "Unicode bidi (sort of) variant b",
+    comment   = "Unicode bidi (sort of) variant c",
 }
 
--- This is a follow up on typo-uba which itself is a follow up on t-bidi by Khaled Hosny which
+-- This is a follow up on typo-uda which itself is a follow up on t-bidi by Khaled Hosny which
 -- in turn is based on minibidi.c from Arabeyes. This is a further optimizations, as well as
 -- an update on some recent unicode bidi developments. There is (and will) also be more control
 -- added. As a consequence this module is somewhat slower than its precursor which itself is
 -- slower than the one-pass bidi handler. This is also a playground and I might add some plugin
 -- support. However, in the meantime performance got a bit better and this third variant is again
--- some 10% faster than the two variant.
+-- some 10% faster than the second variant.
 
 -- todo (cf html):
 --
@@ -56,36 +56,33 @@ local mirrordata          = characters.mirrors
 local textclassdata       = characters.textclasses
 
 local nuts                = nodes.nuts
-local tonut               = nuts.tonut
-local tonode              = nuts.tonode
 
 local getnext             = nuts.getnext
 local getid               = nuts.getid
 local getsubtype          = nuts.getsubtype
-local getchar             = nuts.getchar
 local getlist             = nuts.getlist
+local getchar             = nuts.getchar
 local getattr             = nuts.getattr
-local getfield            = nuts.getfield
 local getprop             = nuts.getprop
-local getdir              = nuts.getdir
+local getdirection        = nuts.getdirection
+local isglyph             = nuts.isglyph
 
-local setfield            = nuts.setfield
 local setprop             = nuts.setprop
 local setchar             = nuts.setchar
-local setdir              = nuts.setdir
+local setdirection        = nuts.setdirection
 local setattrlist         = nuts.setattrlist
 
-local properties          = nodes.properties
+local properties          = nodes.properties.data
 
 local remove_node         = nuts.remove
 local insert_node_after   = nuts.insert_after
 local insert_node_before  = nuts.insert_before
 
 local nodepool            = nuts.pool
-local new_textdir         = nodepool.textdir
+local new_direction       = nodepool.direction
 
 local nodecodes           = nodes.nodecodes
-local skipcodes           = nodes.skipcodes
+local gluecodes           = nodes.gluecodes
 
 local glyph_code          = nodecodes.glyph
 local glue_code           = nodecodes.glue
@@ -94,24 +91,29 @@ local vlist_code          = nodecodes.vlist
 local math_code           = nodecodes.math
 local dir_code            = nodecodes.dir
 local localpar_code       = nodecodes.localpar
-local parfillskip_code    = skipcodes.parfillskip
 
-local maximum_stack       = 0xFF -- unicode: 60, will be jumped to 125, we don't care too much
+local parfillskip_code    = gluecodes.parfillskip
+
+local dirvalues           = nodes.dirvalues
+local lefttoright_code    = dirvalues.lefttoright
+local righttoleft_code    = dirvalues.righttoleft
+
+local maximum_stack       = 0xFF
+
+local a_directions        = attributes.private('directions')
 
 local directions          = typesetters.directions
 local setcolor            = directions.setcolor
 local getfences           = directions.getfences
 
-local a_directions        = attributes.private('directions')
-
 local remove_controls     = true  directives.register("typesetters.directions.removecontrols",function(v) remove_controls  = v end)
 ----- analyze_fences      = true  directives.register("typesetters.directions.analyzefences", function(v) analyze_fences   = v end)
 
-local trace_directions    = false trackers  .register("typesetters.directions.two",           function(v) trace_directions = v end)
-local trace_details       = false trackers  .register("typesetters.directions.two.details",   function(v) trace_details    = v end)
-local trace_list          = false trackers  .register("typesetters.directions.two.list",      function(v) trace_list       = v end)
+local report_directions   = logs.reporter("typesetting","directions three")
 
-local report_directions   = logs.reporter("typesetting","directions two")
+local trace_directions    = false trackers.register("typesetters.directions",         function(v) trace_directions = v end)
+local trace_details       = false trackers.register("typesetters.directions.details", function(v) trace_details    = v end)
+local trace_list          = false trackers.register("typesetters.directions.list",    function(v) trace_list       = v end)
 
 -- strong (old):
 --
@@ -122,7 +124,7 @@ local report_directions   = logs.reporter("typesetting","directions two")
 -- lre : left to right embedding
 -- rle : left to left embedding
 -- al  : right to legt arabic (esp punctuation issues)
-
+--
 -- weak:
 --
 -- en  : english number
@@ -132,23 +134,23 @@ local report_directions   = logs.reporter("typesetting","directions two")
 -- cs  : common number separator
 -- nsm : nonspacing mark
 -- bn  : boundary neutral
-
+--
 -- neutral:
 --
 -- b  : paragraph separator
 -- s  : segment separator
 -- ws : whitespace
 -- on : other neutrals
-
+--
 -- interesting: this is indeed better (and more what we expect i.e. we already use this split
 -- in the old original (also these isolates)
-
+--
 -- strong (new):
 --
 -- l   : left to right
 -- r   : right to left
 -- al  : right to left arabic (esp punctuation issues)
-
+--
 -- explicit: (new)
 --
 -- lro : left to right override
@@ -211,27 +213,28 @@ end
 local function show_done(list,size)
     local joiner = utfchar(0x200C)
     local result = { }
+    local format = formatters["<%s>"]
     for i=1,size do
         local entry     = list[i]
         local character = entry.char
         local begindir  = entry.begindir
         local enddir    = entry.enddir
         if begindir then
-            result[#result+1] = formatters["<%s>"](begindir)
+            result[#result+1] = format(begindir)
         end
         if entry.remove then
             -- continue
         elseif character == 0xFFFC then
-            result[#result+1] = formatters["<%s>"]("?")
+            result[#result+1] = format("?")
         elseif character == 0x0020 then
-            result[#result+1] = formatters["<%s>"](" ")
+            result[#result+1] = format(" ")
         elseif character >= 0x202A and character <= 0x202C then
-            result[#result+1] = formatters["<%s>"](entry.original)
+            result[#result+1] = format(entry.original)
         else
             result[#result+1] = utfchar(character)
         end
         if enddir then
-            result[#result+1] = formatters["<%s>"](enddir)
+            result[#result+1] = format(enddir)
         end
     end
     return concat(result,joiner)
@@ -240,32 +243,35 @@ end
 -- keeping the list and overwriting doesn't save much runtime, only a few percent
 -- char is only used for mirror, so in fact we can as well only store it for
 -- glyphs only
-
+--
 -- tracking what direction is used and skipping tests is not faster (extra kind of
 -- compensates gain)
 
-local mt_space  = { __index = { char = 0x0020, direction = "ws",  original = "ws",  level = 0 } }
-local mt_lre    = { __index = { char = 0x202A, direction = "lre", original = "lre", level = 0 } }
-local mt_lre    = { __index = { char = 0x202B, direction = "rle", original = "rle", level = 0 } }
-local mt_pdf    = { __index = { char = 0x202C, direction = "pdf", original = "pdf", level = 0 } }
-local mt_object = { __index = { char = 0xFFFC, direction = "on",  original = "on",  level = 0 } }
+local mt_space  = { __index = { char = 0x0020, direction = "ws",  original = "ws",  level = 0, skip = 0 } }
+local mt_lre    = { __index = { char = 0x202A, direction = "lre", original = "lre", level = 0, skip = 0 } }
+local mt_rle    = { __index = { char = 0x202B, direction = "rle", original = "rle", level = 0, skip = 0 } }
+local mt_pdf    = { __index = { char = 0x202C, direction = "pdf", original = "pdf", level = 0, skip = 0 } }
+local mt_object = { __index = { char = 0xFFFC, direction = "on",  original = "on",  level = 0, skip = 0 } }
 
-local function build_list(head) -- todo: store node pointer ... saves loop
+local stack = table.setmetatableindex("table") -- shared
+local list  = { }                              -- shared
+
+local function build_list(head,where)
     -- P1
     local current = head
-    local list    = { }
     local size    = 0
     while current do
         size = size + 1
         local id = getid(current)
         local p  = properties[current]
-        local t
         if p and p.directions then
+            -- tricky as dirs can be injected in between
             local skip = 0
             local last = id
             current    = getnext(current)
             while current do
                 local id = getid(current)
+                local p  = properties[current]
                 if p and p.directions then
                     skip    = skip + 1
                     last    = id
@@ -275,35 +281,28 @@ local function build_list(head) -- todo: store node pointer ... saves loop
                 end
             end
             if id == last then -- the start id
-                t = { skip = skip, id = id }
+                list[size] = setmetatable({ skip = skip, id = id },mt_object)
             else
-                t = { skip = skip, id = id, last = last }
+                list[size] = setmetatable({ skip = skip, id = id, last = last },mt_object)
             end
-            setmetatable(t,mt_object)
         elseif id == glyph_code then
-            local chr = getchar(current)
-            local dir = directiondata[chr]
-            t = { char = chr, direction = dir, original = dir, level = 0 }
-            current = getnext(current)
+            local chr  = getchar(current)
+            local dir  = directiondata[chr]
+            -- could also be a metatable
+            list[size] = { char = chr, direction = dir, original = dir, level = 0 }
+            current    = getnext(current)
          -- if not list[dir] then list[dir] = true end -- not faster when we check for usage
         elseif id == glue_code then -- and how about kern
-            t = { }
-            setmetatable(t,mt_space)
-            current = getnext(current)
+            list[size] = setmetatable({ },mt_space)
+            current    = getnext(current)
         elseif id == dir_code then
-            local dir = getdir(current)
-            if dir == "+TLT" then
-                t = { }
-                setmetatable(t,mt_lre)
-            elseif dir == "+TRT" then
-                t = { }
-                setmetatable(t,mt_rle)
-            elseif dir == "-TLT" or dir == "-TRT" then
-                t = { }
-                setmetatable(t,mt_pdf)
+            local dir, pop = getdirection(current)
+            if dir == lefttoright_code then
+                list[size] = setmetatable({ },pop and mt_pdf or mt_lre)
+            elseif dir == righttoleft_code then
+                list[size] = setmetatable({ },pop and mt_pdf or mt_rle)
             else
-                t = { id = id }
-                setmetatable(t,mt_object)
+                list[size] = setmetatable({ id = id },mt_object)
             end
             current = getnext(current)
         elseif id == math_code then
@@ -315,9 +314,8 @@ local function build_list(head) -- todo: store node pointer ... saves loop
             end
             skip       = skip + 1
             current    = getnext(current)
-            t = { id = id, skip = skip }
-            setmetatable(t,mt_object)
-        else
+            list[size] = setmetatable({ id = id, skip = skip },mt_object)
+        else -- disc_code: we assume that these are the same as the surrounding
             local skip = 0
             local last = id
             current    = getnext(current)
@@ -331,16 +329,12 @@ local function build_list(head) -- todo: store node pointer ... saves loop
                     break
                 end
             end
-            if skip == 0 then
-                t = { id = id }
-            elseif id == last then -- the start id
-                t = { id = id, skip = skip }
+            if id == last then -- the start id
+                list[size] = setmetatable({ id = id, skip = skip },mt_object)
             else
-                t = { id = id, skip = skip, last = last }
+                list[size] = setmetatable({ id = id, skip = skip, last = last },mt_object)
             end
-            setmetatable(t,mt_object)
         end
-        list[size] = t
     end
     return list, size
 end
@@ -359,10 +353,11 @@ end
 -- ש ( ל ( א ) כ ) 2-8,4-6
 -- ש ( ל [ א ] כ ) 2-8,4-6
 
+local fencestack = table.setmetatableindex("table")
+
 local function resolve_fences(list,size,start,limit)
     -- N0: funny effects, not always better, so it's an option
-    local stack = { }
-    local top   = 0
+    local nofstack = 0
     for i=start,limit do
         local entry = list[i]
         if entry.direction == "on" then
@@ -373,15 +368,17 @@ local function resolve_fences(list,size,start,limit)
                 entry.mirror = mirror
                 entry.class  = class
                 if class == "open" then
-                    top = top + 1
-                    stack[top] = { mirror, i, false }
-                elseif top == 0 then
+                    nofstack       = nofstack + 1
+                    local stacktop = fencestack[nofstack]
+                    stacktop[1]    = mirror
+                    stacktop[2]    = i
+                elseif nofstack == 0 then
                     -- skip
                 elseif class == "close" then
-                    while top > 0 do
-                        local s = stack[top]
-                        if s[1] == char then
-                            local open  = s[2]
+                    while nofstack > 0 do
+                        local stacktop = fencestack[nofstack]
+                        if stacktop[1] == char then
+                            local open  = stacktop[2]
                             local close = i
                             list[open ].paired = close
                             list[close].paired = open
@@ -389,7 +386,7 @@ local function resolve_fences(list,size,start,limit)
                         else
                             -- do we mirror or not
                         end
-                        top = top - 1
+                        nofstack = nofstack - 1
                     end
                 end
             end
@@ -412,27 +409,32 @@ end
 
 -- the action
 
-local function get_baselevel(head,list,size) -- todo: skip if first is object (or pass head and test for localpar)
-    local id = getid(head)
-    if id == localpar_code then
-        if getdir(head) == "TRT" then
-            return 1, "TRT", true
-        else
-            return 0, "TLT", true
+local function get_baselevel(head,list,size,direction)
+    if direction == lefttoright_code or direction == righttoleft_code then
+        return direction, true
+    elseif getid(head) == localpar_code and getsubtype(head) == 0 then
+        direction = getdirection(head)
+        if direction == lefttoright_code or direction == righttoleft_code then
+            return direction, true
         end
-    else
-        -- P2, P3
-        for i=1,size do
-            local entry     = list[i]
-            local direction = entry.direction
-            if direction == "r" or direction == "al" then -- and an ?
-                return 1, "TRT", true
-            elseif direction == "l" then
-                return 0, "TLT", true
-            end
-        end
-        return 0, "TLT", false
     end
+    -- for old times sake we we handle strings too
+    if direction == "TLT" then
+        return lefttoright_code, true
+    elseif direction == "TRT" then
+        return righttoleft_code, true
+    end
+    -- P2, P3
+    for i=1,size do
+        local entry     = list[i]
+        local direction = entry.direction
+        if direction == "r" or direction == "al" then -- and an ?
+            return righttoleft_code, true
+        elseif direction == "l" then
+            return lefttoright_code, true
+        end
+    end
+    return lefttoright_code, false
 end
 
 local function resolve_explicit(list,size,baselevel)
@@ -440,7 +442,6 @@ local function resolve_explicit(list,size,baselevel)
     -- X1
     local level    = baselevel
     local override = "on"
-    local stack    = { }
     local nofstack = 0
     for i=1,size do
         local entry     = list[i]
@@ -449,7 +450,9 @@ local function resolve_explicit(list,size,baselevel)
         if direction == "rle" then
             if nofstack < maximum_stack then
                 nofstack        = nofstack + 1
-                stack[nofstack] = { level, override }
+                local stacktop  = stack[nofstack]
+                stacktop[1]     = level
+                stacktop[2]     = override
                 level           = level + (level % 2 == 1 and 2 or 1) -- least_greater_odd(level)
                 override        = "on"
                 entry.level     = level
@@ -462,7 +465,9 @@ local function resolve_explicit(list,size,baselevel)
         elseif direction == "lre" then
             if nofstack < maximum_stack then
                 nofstack        = nofstack + 1
-                stack[nofstack] = { level, override }
+                local stacktop  = stack[nofstack]
+                stacktop[1]     = level
+                stacktop[2]     = override
                 level           = level + (level % 2 == 1 and 1 or 2) -- least_greater_even(level)
                 override        = "on"
                 entry.level     = level
@@ -475,7 +480,9 @@ local function resolve_explicit(list,size,baselevel)
         elseif direction == "rlo" then
             if nofstack < maximum_stack then
                 nofstack        = nofstack + 1
-                stack[nofstack] = { level, override }
+                local stacktop  = stack[nofstack]
+                stacktop[1]     = level
+                stacktop[2]     = override
                 level           = level + (level % 2 == 1 and 2 or 1) -- least_greater_odd(level)
                 override        = "r"
                 entry.level     = level
@@ -488,7 +495,9 @@ local function resolve_explicit(list,size,baselevel)
         elseif direction == "lro" then
             if nofstack < maximum_stack then
                 nofstack        = nofstack + 1
-                stack[nofstack] = { level, override }
+                local stacktop  = stack[nofstack]
+                stacktop[1]     = level
+                stacktop[2]     = override
                 level           = level + (level % 2 == 1 and 1 or 2) -- least_greater_even(level)
                 override        = "l"
                 entry.level     = level
@@ -499,16 +508,20 @@ local function resolve_explicit(list,size,baselevel)
             end
         -- X7
         elseif direction == "pdf" then
-            if nofstack < maximum_stack then
+            if nofstack > 0 then
                 local stacktop  = stack[nofstack]
-                nofstack        = nofstack - 1
                 level           = stacktop[1]
                 override        = stacktop[2]
+                nofstack        = nofstack - 1
                 entry.level     = level
                 entry.direction = "bn"
                 entry.remove    = true
             elseif trace_directions then
-                report_directions("stack overflow at position %a with direction %a",i,direction)
+                report_directions("stack underflow at position %a with direction %a",
+                    i, direction)
+            else
+                report_directions("stack underflow at position %a with direction %a: %s",
+                    i, direction, show_list(list,size))
             end
         -- X6
         else
@@ -518,11 +531,6 @@ local function resolve_explicit(list,size,baselevel)
             end
         end
     end
--- else
---     for i=1,size do
---         list[i].level = baselevel
---     end
--- end
     -- X8 (reset states and overrides after paragraph)
 end
 
@@ -591,29 +599,31 @@ local function resolve_weak(list,size,start,limit,orderbefore,orderafter)
         end
     else -- only more efficient when we have es/cs
         local runner = start + 2
-        local before = list[start]
-        local entry  = list[start + 1]
-        local after  = list[runner]
-        while after do
-            local direction = entry.direction
-            if direction == "es" then
-                if before.direction == "en" and after.direction == "en" then
-                    entry.direction = "en"
-                end
-            elseif direction == "cs" then
-                local prevdirection = before.direction
-                if prevdirection == "en" then
-                    if after.direction == "en" then
+        if runner <= limit then
+            local before = list[start]
+            local entry  = list[start + 1]
+            local after  = list[runner]
+            while after do
+                local direction = entry.direction
+                if direction == "es" then
+                    if before.direction == "en" and after.direction == "en" then
                         entry.direction = "en"
                     end
-                elseif prevdirection == "an" and after.direction == "an" then
-                    entry.direction = "an"
+                elseif direction == "cs" then
+                    local prevdirection = before.direction
+                    if prevdirection == "en" then
+                        if after.direction == "en" then
+                            entry.direction = "en"
+                        end
+                    elseif prevdirection == "an" and after.direction == "an" then
+                        entry.direction = "an"
+                    end
                 end
+                before  = current
+                current = after
+                after   = list[runner]
+                runner  = runner + 1
             end
-            before  = current
-            current = after
-            after   = list[runner]
-            runner  = runner + 1
         end
     end
 -- end
@@ -724,33 +734,6 @@ local function resolve_neutral(list,size,start,limit,orderbefore,orderafter)
     end
 end
 
--- local function resolve_implicit(list,size,start,limit,orderbefore,orderafter)
---     -- I1
---     for i=start,limit do
---         local entry = list[i]
---         local level = entry.level
---         if level % 2 ~= 1 then -- not odd(level)
---             local direction = entry.direction
---             if direction == "r" then
---                 entry.level = level + 1
---             elseif direction == "an" or direction == "en" then
---                 entry.level = level + 2
---             end
---         end
---     end
---     -- I2
---     for i=start,limit do
---         local entry = list[i]
---         local level = entry.level
---         if level % 2 == 1 then -- odd(level)
---             local direction = entry.direction
---             if direction == "l" or direction == "en" or direction == "an" then
---                 entry.level = level + 1
---             end
---         end
---     end
--- end
-
 local function resolve_implicit(list,size,start,limit,orderbefore,orderafter,baselevel)
     for i=start,limit do
         local entry     = list[i]
@@ -850,11 +833,13 @@ local function resolve_levels(list,size,baselevel,analyze_fences)
     end
 end
 
+local stack = { }
+
 local function insert_dir_points(list,size)
     -- L2, but no actual reversion is done, we simply annotate where
     -- begindir/endddir node will be inserted.
     local maxlevel = 0
-    local finaldir = false
+    local toggle   = true
     for i=1,size do
         local level = list[i].level
         if level > maxlevel then
@@ -862,15 +847,18 @@ local function insert_dir_points(list,size)
         end
     end
     for level=0,maxlevel do
-        local started  = false
-        local begindir = nil
-        local enddir   = nil
-        if level % 2 == 1 then
-            begindir = "+TRT"
-            enddir   = "-TRT"
+        local started  -- = false
+        local begindir -- = nil
+        local enddir   -- = nil
+        local prev     -- = nil
+        if toggle then
+            begindir = lefttoright_code
+            enddir   = lefttoright_code
+            toggle   = false
         else
-            begindir = "+TLT"
-            enddir   = "-TLT"
+            begindir = righttoleft_code
+            enddir   = righttoleft_code
+            toggle   = true
         end
         for i=1,size do
             local entry = list[i]
@@ -881,25 +869,45 @@ local function insert_dir_points(list,size)
                 end
             else
                 if started then
-                    list[i-1].enddir = enddir
-                    started          = false
+                    prev.enddir = enddir
+                    started     = false
                 end
             end
-        end
-        -- make sure to close the run at end of line
-        if started then
-            finaldir = enddir
+            prev = entry
         end
     end
-    if finaldir then
-        list[size].enddir = finaldir
+    -- make sure to close the run at end of line
+    local last = list[size]
+    if not last.enddir then
+        local n = 0
+        for i=1,size do
+            local entry = list[i]
+            local e = entry.enddir
+            local b = entry.begindir
+            if e then
+                n = n - 1
+            end
+            if b then
+                n = n + 1
+                stack[n] = b
+            end
+        end
+        if n > 0 then
+            if trace_list and n > 1 then
+                report_directions("unbalanced list")
+            end
+            last.enddir = stack[n]
+        end
     end
 end
 
+-- We flag nodes that can be skipped when we see them again but because whatever
+-- mechanism can injetc dir nodes that then are not flagged, we don't flag dir
+-- nodes that we inject here.
+
 local function apply_to_list(list,size,head,pardir)
     local index   = 1
     local current = head
-    local done    = false
     if trace_list then
         report_directions("start run")
     end
@@ -912,7 +920,12 @@ local function apply_to_list(list,size,head,pardir)
         local entry    = list[index]
         local begindir = entry.begindir
         local enddir   = entry.enddir
-        local p = properties[current] if p then p.directions = true else properties[current] = { directions = true } end
+        local p = properties[current]
+        if p then
+            p.directions = true
+        else
+            properties[current] = { directions = true }
+        end
         if id == glyph_code then
             local mirror = entry.mirror
             if mirror then
@@ -933,55 +946,43 @@ local function apply_to_list(list,size,head,pardir)
                 setcolor(current,direction,false,mirror)
             end
         elseif id == hlist_code or id == vlist_code then
-            setdir(current,pardir) -- is this really needed?
+            setdirection(current,pardir) -- is this really needed?
         elseif id == glue_code then
             if enddir and getsubtype(current) == parfillskip_code then
                 -- insert the last enddir before \parfillskip glue
-                local d = new_textdir(enddir)
-                local p = properties[d] if p then p.directions = true else properties[d] = { directions = true } end
-             -- setattrlist(d,current)
-                head = insert_node_before(head,current,d)
+                head = insert_node_before(head,current,new_direction(enddir,true))
                 enddir = false
-                done = true
             end
         elseif begindir then
-            if id == localpar_code then
+            if id == localpar_code and getsubtype(current) == 0 then
                 -- localpar should always be the 1st node
-                local d = new_textdir(begindir)
-                local p = properties[d] if p then p.directions = true else properties[d] = { directions = true } end
-             -- setattrlist(d,current)
-                head, current = insert_node_after(head,current,d)
+                head, current = insert_node_after(head,current,new_direction(begindir))
                 begindir = nil
-                done = true
             end
         end
         if begindir then
-            local d = new_textdir(begindir)
-            local p = properties[d] if p then p.directions = true else properties[d] = { directions = true } end
-         -- setattrlist(d,current)
-            head = insert_node_before(head,current,d)
-            done = true
+            head = insert_node_before(head,current,new_direction(begindir))
         end
         local skip = entry.skip
         if skip and skip > 0 then
             for i=1,skip do
                 current = getnext(current)
-                local p = properties[current] if p then p.directions = true else properties[current] = { directions = true } end
+                local p = properties[current]
+                if p then
+                    p.directions = true
+                else
+                    properties[current] = { directions = true }
+                end
             end
         end
         if enddir then
-            local d = new_textdir(enddir)
-            local p = properties[d] if p then p.directions = true else properties[d] = { directions = true } end
-         -- setattrlist(d,current)
-            head, current = insert_node_after(head,current,d)
-            done = true
+            head, current = insert_node_after(head,current,new_direction(enddir,true))
         end
         if not entry.remove then
             current = getnext(current)
         elseif remove_controls then
             -- X9
             head, current = remove_node(head,current,true)
-            done = true
         else
             current = getnext(current)
         end
@@ -990,21 +991,24 @@ local function apply_to_list(list,size,head,pardir)
     if trace_list then
         report_directions("stop run")
     end
-    return head, done
+    return head
 end
 
-local function process(head)
-    head = tonut(head)
+-- If needed we can optimize for only_one. There is no need to do anything
+-- when it's not a glyph. Otherwise we only need to check mirror and apply
+-- directions when it's different from the surrounding. Paragraphs always
+-- have more than one node. Actually, we only enter this function when we
+-- do have a glyph!
+
+local function process(head,direction,only_one,where)
     -- for the moment a whole paragraph property
     local attr = getattr(head,a_directions)
     local analyze_fences = getfences(attr)
     --
-    local list, size = build_list(head)
-    local baselevel, pardir, dirfound = get_baselevel(head,list,size) -- we always have an inline dir node in context
-    if not dirfound and trace_details then
-        report_directions("no initial direction found, gambling")
-    end
+    local list, size = build_list(head,where)
+    local baselevel, dirfound = get_baselevel(head,list,size,direction)
     if trace_details then
+        report_directions("analyze: baselevel %a",baselevel == righttoleft_code and "r2l" or "l2r")
         report_directions("before : %s",show_list(list,size,"original"))
     end
     resolve_explicit(list,size,baselevel)
@@ -1014,8 +1018,12 @@ local function process(head)
         report_directions("after  : %s",show_list(list,size,"direction"))
         report_directions("result : %s",show_done(list,size))
     end
-    head, done = apply_to_list(list,size,head,pardir)
-    return tonode(head), done
+    return apply_to_list(list,size,head,baselevel)
 end
 
-directions.installhandler(interfaces.variables.three,process)
+local variables = interfaces.variables
+
+directions.installhandler(variables.one,    process) -- for old times sake
+directions.installhandler(variables.two,    process) -- for old times sake
+directions.installhandler(variables.three,  process) -- for old times sake
+directions.installhandler(variables.unicode,process)
