@@ -7,7 +7,7 @@
 -----------------------------------------------------------------------
 
 if not modules then modules = { } end modules ['xindex-lib'] = {
-      version = 0.07,
+      version = 0.18,
       comment = "main library  to xindex.lua",
        author = "Herbert Voss",
     copyright = "Herbert Voss",
@@ -19,16 +19,24 @@ local escapechar2 = "\"\""
 local Letters = "ÄÖÜäöüabcdefghijklmnopqrstuvwxyzßABCDEFGHIJKLMNOPQRSTUVWXYZ"
 local Digits = "0123456789"
 
--- Looks up the character `character´ in the alphabet and returns its 'normalization' for sorting
-local function get_normalized_char( CharList, character )
+function CreateCharListMap(CharList)
+  local map = {}
   for i, alphabet_entry in ipairs(CharList) do
-    for _, alphabet_char in ipairs(alphabet_entry) do
-      if character == alphabet_char then
-        return alphabet_entry[1]
+    local first_char
+    for j, alphabet_char in ipairs(alphabet_entry) do
+      if j == 1 then
+        first_char = alphabet_char
+      else
+        map[alphabet_char] = first_char
       end
     end
   end
-  return character
+  return map
+end
+
+-- Looks up the character `character´ in the alphabet and returns its 'normalization' for sorting
+local function get_normalized_char( CharListMap, character )
+  return CharListMap[character] or character
 end
 
 function Lower(strOrig)   -- return UTF string.lower
@@ -47,27 +55,61 @@ end
 function NormalizedLower(strOrig)  -- return normalized UTF string.lower (ä -> a)
   local str = ""
   for i=1, utf.len(strOrig) do
-    local c = get_normalized_char(alphabet_lower,utf.sub(strOrig,i,i))
+    local c = get_normalized_char(alphabet_lower_map,utf.sub(strOrig,i,i))
     str = str..string.lower(c)
   end
   return str
 end
 
 function NormalizedUpper(strOrig)  -- return normalized UTF string.upper (ä -> A)
-  local str = ""
+  local str = {}
+  local c
   for i=1, utf.len(strOrig) do
-    local c = get_normalized_char(alphabet_lower,utf.sub(strOrig,i,i))
-    str = str..string.upper(c)
+    c = get_normalized_char(alphabet_lower_map,utf.sub(strOrig,i,i))
+    str[#str + 1] = c
   end
-  strOrig = str
-  str = ""
+  strOrig = table.concat(str)
+  str = {}
   for i=1, utf.len(strOrig) do
-    local c = get_normalized_char(alphabet_upper,utf.sub(strOrig,i,i))
-    str = str..string.upper(c)
+    c = get_normalized_char(alphabet_upper_map,utf.sub(strOrig,i,i))
+    c = NormalizedUppercase(c)
+    str[#str + 1] = c
   end
-  return str
+  return table.concat(str)
 end
 
+function NormalizedUppercase(chr)  -- return normalized UTF uppercase letter
+  return alphabet_uppercase_map[chr] or chr
+end
+
+function NormalizedChars(strOrig)  -- return normalized UTF string (ä -> a) with first as upper case
+  local c = get_normalized_char(alphabet_lower_map,utf.sub(strOrig,1,1))
+  c = get_normalized_char(alphabet_upper_map, c)
+  c = string.upper(c)
+  local str = { c }
+  for ch in string.utfcharacters(utf.sub(strOrig, 2)) do
+    c = get_normalized_char(alphabet_lower_map, ch)
+    c = get_normalized_char(alphabet_upper_map, c)
+    str[#str + 1] = c
+  end
+  return table.concat(str)
+end
+  
+--  local strOrigTrue = strOrig
+--  strOrig = table.concat(str)
+--  c = string.upper(get_normalized_char(alphabet_upper,utf.sub(strOrig,1,1)))
+--  str = { c }
+--  for i=2, utf.len(strOrig) do
+--    c = get_normalized_char(alphabet_upper,utf.sub(strOrig,i,i))
+--    str[#str + 1] = c
+--  end
+--  for ch in string.utfcharacters(strOrig, 2) do
+--    str[#str + 1] = get_normalized_char(alphabet_upper, ch)
+--  end
+--  print(strOrigTrue.." -> "..str)
+--  return table.concat(str)
+--end
+
 function checkVert(str) -- get the | part
   local vert = string.find(str,"|",1,true)
   if (vert) then 
@@ -162,7 +204,7 @@ function findSequences(a) -- look for 1,2,3,4, ...
     if a[i]-a[i-1] > 1 then  			-- not next page
       lastPage = a[i-1]      			-- new start for a sequence search
       if lastPage - firstPage >= minCompress then 		-- sequence found
-        b[#b+1]= tostring(firstPage).."--"..tostring(lastPage)
+        b[#b+1]= tostring(firstPage)..rangeSymbol..tostring(lastPage)
       else
         b[#b+1]= firstPage
         if (firstPage ~= lastPage) then b[#b+1]= lastPage end
@@ -171,7 +213,7 @@ function findSequences(a) -- look for 1,2,3,4, ...
     end
   end
   if a[#a] - firstPage > 2 then  -- test file end
-    b[#b+1]= tostring(firstPage).."--"..tostring(a[#a])
+    b[#b+1]= tostring(firstPage)..rangeSymbol..tostring(a[#a])
   else
     b[#b+1]= firstPage
     if (firstPage ~= a[#a]) then b[#b+1]= a[#a] end
@@ -206,7 +248,7 @@ function checkParenthesis(Entries) -- search for |( ... |) entries
       elseif paraClose then
         lastPage = v["pages"][1]["number"]
         newEntries[#newEntries+1] = { Entry   = v["Entry"],
-                                      pages   = {{ number  = firstPage.."--"..lastPage,
+                                      pages   = {{ number  = firstPage..rangeSymbol..lastPage,
                                                   special = deleteOpenClose(v["pages"][1]["special"])}},
                                       sortChar= v["sortChar"],
                                       Macro   = v["Macro"]
@@ -339,10 +381,14 @@ function compressPageList(pages) --  called with (v["pages"])
   end  -- only one pageno
   if #pages == 2 then  -- only two pages
     if tonumber(pages[1]["number"]) and tonumber(pages[2]["number"]) then
-      if ((tonumber(pages[2]["number"]) - tonumber(pages[1]["number"])) == 1) and 
-          (pages[1]["special"] == pages[2]["special"]) then 
-        pages[1]["number"] = pages[1]["number"].."f"  
-        pages[2] = nil
+      if ((tonumber(pages[2]["number"]) - tonumber(pages[1]["number"])) == 1) and (pages[1]["special"] == pages[2]["special"]) then 
+        if fCompress then
+          pages[1]["number"] = pages[1]["number"]..page_folium[1]  
+          pages[2] = nil
+        elseif minCompress == 1 then
+          pages[1]["number"] = pages[1]["number"]..rangeSymbol..pages[2]["number"]  
+          pages[2] = nil
+        end
 --      elseif (pages[1]["number"] == pages[2]["number"]) and
 --             (pages[1]["special"] == pages[2]["special"]) then 
 --        pages[2] = nil
@@ -355,7 +401,7 @@ function compressPageList(pages) --  called with (v["pages"])
   local startIndex
   local newPages = {}  -- { pages[1], pages[2], ... }
   local series = {}    -- { pages[1], pages[2], ... }
-  if tonumber(pages[1]["number"]) then -- if a number we start with seconf page
+  if tonumber(pages[1]["number"]) then -- if a number we start with second page
     newPages = {}
     series = {pages[1]}
     startIndex = 2
@@ -373,9 +419,9 @@ if args_v then   print ("compressPageList: more than two pages for the entry\n")
       elseif (#series == 1) then    --     no -f page -> only one page -> output
         newPages[#newPages+1] = series[1]
         series = {pages[i]}
-      elseif (#series > minCompress) then  -- we found series od f pages like 11, 12, 13, 14 
+      elseif (#series > minCompress) then  -- we found series or f pages like 11, 12, 13, 14 
         -- the sequenz gets the special from the first page
-        newPages[#newPages+1] = { number = series[1]["number"].."--"..series[#series]["number"], special = series[1]["special"]}  -- first..last
+        newPages[#newPages+1] = { number = series[1]["number"]..rangeSymbol..series[#series]["number"], special = series[1]["special"]}  -- first..last
         series = {pages[i]}
       else -- series < minCompress
         if fCompress then
@@ -414,7 +460,7 @@ if args_v then   print ("compressPageList: more than two pages for the entry\n")
         series = {pages[i]}
       else    --  current page is non numeric
         if (#series > minCompress) then  -- we found a page series
-          newPages[#newPages+1] = { number = series[1]["number"].."--"..series[#series]["number"], -- first..last
+          newPages[#newPages+1] = { number = series[1]["number"]..rangeSymbol..series[#series]["number"], -- first..last
                                     special =series[1]["special"] }
           series = {pages[i]}
         else -- series < minCompress
@@ -466,7 +512,7 @@ function get_category(code_point)
 end
 
 function getCharType(c)
---  print ("getCharType c="..c..": codepoint="..utf8.codepoint(c))
+--  print ("getCharType c="..tostring(c).." "..type(c)..": codepoint="..utf8.codepoint(c))
   local category = get_category(utf8.codepoint(c))
 --  print ("getCharType: "..category)
   if category == "Nd" then return 1 
@@ -489,29 +535,42 @@ end
 -- str = "\indexentry {foo}{bar}"
 -- first, second = str:match('{([^}]*)}%s*{([^}]*)}')  -> foo, bar
 
-function UTFCompare(a,b)  
--- a, b are something like \indexentry{foo}{bar}
---  writeLog(1,"UTFCompare:  "..a["Entry"]..", "..a["pages"][1]["number"].." - "..b["Entry"]..", "..b["pages"][1]["number"].."\n",2)
---  k = k + 1
---  if (k % 50) == 0 then writeLog(1,".",1) end
-  local A,B,Apage,Bpage
+function GenerateSortKey(a)
+  local A, Apage
   if numericPage then
     if tonumber(a["pages"][1]["number"]) then
       Apage = string.format("%09d",a["pages"][1]["number"])
     else
       Apage = string.format("%09d",romanToNumber(a["pages"][1]["number"]))
     end
-    if tonumber(b["pages"][1]["number"]) then
-      Bpage = string.format("%09d",b["pages"][1]["number"])
-    else
-      Bpage = string.format("%09d",romanToNumber(b["pages"][1]["number"]))
-    end
   else
     Apage = string.format("%09s",a["pages"][1]["number"])
-    Bpage = string.format("%09s",b["pages"][1]["number"])
   end
-  A = NormalizedUpper(getSortString(a["Entry"].." "..Apage..a["pages"][1]["special"]):gsub('!','')) -- replace! by empty
-  B = NormalizedUpper(getSortString(b["Entry"].." "..Bpage..b["pages"][1]["special"]):gsub('!',''))
+  if no_caseSensitive then
+    A = NormalizedUpper(getSortString(a["Entry"].." "..Apage..a["pages"][1]["special"]):gsub('!','')) -- replace! by empty
+    a["SortKey"] = A
+  else  
+    A = NormalizedChars(getSortString(a["Entry"].." "..Apage..a["pages"][1]["special"]):gsub('!','')) -- replace! by empty
+    a["SortKey"] = A
+  end
+  -- return A
+end
+
+function GenerateSortKeys(Index)
+  for _, entry in ipairs(Index) do
+    GenerateSortKey(entry)
+  end
+end
+
+function UTFCompare(a,b)  
+-- a, b are something like \indexentry{foo}{bar}
+--  writeLog(1,"UTFCompare:  "..a["Entry"]..", "..a["pages"][1]["number"].." - "..b["Entry"]..", "..b["pages"][1]["number"].."\n",2)
+--  k = k + 1
+--  if (k % 50) == 0 then writeLog(1,".",1) end
+
+  local A = a["SortKey"]
+  local B = b["SortKey"]
+  
   writeLog(1,"UTFCompare: A--B "..A.."--"..B.."\n",2)
 -- print(A,B)
 --[[ 
@@ -562,8 +621,8 @@ function pageCompare(a,b)  -- a = {{number=...,special=..},{...,...}}
     -- now we have some special page numbers 
     A = tostring(a0)
     B = tostring(b0)
-    A1 = A:find("--",1,true) or A:find("f",1,true)  -- sequence or folio pages
-    B1 = B:find("--",1,true) or B:find("f",1,true)
+    A1 = A:find(rangeSymbol,1,true) or A:find("f",1,true)  -- sequence or folio pages
+    B1 = B:find(rangeSymbol,1,true) or B:find("f",1,true)
     if A1 then a0 = tonumber(A:sub(1,A1-1)) end
     if B1 then b0 = tonumber(B:sub(1,B1-1)) end
     return tonumber(a0) < tonumber(b0)  -- numeric
@@ -572,6 +631,13 @@ function pageCompare(a,b)  -- a = {{number=...,special=..},{...,...}}
   end
 end
 
+function charCompare(a,b)  -- compare only first char
+  local A,B
+  A = string.upper(utf.sub(a["sortChar"],1,1))
+  B = string.upper(utf.sub(b["sortChar"],1,1))
+  return A < B
+end
+
 function checkFF(p)
   if not p then return p end
   local P = tostring(p)
@@ -692,11 +758,12 @@ function printList(Index,level)
 end
 
 function getItem(str,n)
-  if n >= 0 then 
+ 
+  if (n >= 0) and str then 
     local item = (str:gsub("!.*", "%0!")
                      :gsub("[^!]*!", "", n)
                      :match("^([^!]*)!") ) 
-    writeLog(1,"getItem ("..str..", "..n..") = "..item.."\n",2)
+--    writeLog(1,"getItem ("..str..", "..n..") = "..item.."\n",2)
     return item
   end
 end
@@ -859,7 +926,13 @@ local match = function(expr)
   return Ct(idx):match(expr)
 end
 
+local SortStringCache = {}
 function getSortString(Entry)
+  local sortString = SortStringCache[Entry]
+  if sortString ~= nil then
+    return sortString
+  end
+
   local t = match(Entry)
 --  require 'xindex-pretty'.dump(t)   -- only for internal dump
   local data = t[1]
@@ -872,6 +945,7 @@ function getSortString(Entry)
       data = data..t[i]
     end
   end
+  SortStringCache[Entry] = data
   return data
 end
 
