@@ -22,7 +22,8 @@ local utftoeight = utf.toeight
 local splitter = lpeg.tsplitat(".")
 
 function utilities.tables.definetable(target,nofirst,nolast) -- defines undefined tables
-    local composed, t = nil, {  }
+    local composed = nil
+    local t        = { }
     local snippets = lpegmatch(splitter,target)
     for i=1,#snippets - (nolast and 1 or 0) do
         local name = snippets[i]
@@ -310,23 +311,24 @@ function tables.encapsulate(core,capsule,protect)
     end
 end
 
--- best keep [%q] keys (as we have some in older applications i.e. saving user data
+-- best keep [%q] keys (as we have some in older applications i.e. saving user data (otherwise
+-- we also need to check for reserved words)
 
-local f_hashed_string   = formatters["[%q]=%q,"]
-local f_hashed_number   = formatters["[%q]=%s,"]
-local f_hashed_boolean  = formatters["[%q]=%l,"]
-local f_hashed_table    = formatters["[%q]="]
+local f_hashed_string   = formatters["[%Q]=%Q,"]
+local f_hashed_number   = formatters["[%Q]=%s,"]
+local f_hashed_boolean  = formatters["[%Q]=%l,"]
+local f_hashed_table    = formatters["[%Q]="]
 
-local f_indexed_string  = formatters["[%s]=%q,"]
+local f_indexed_string  = formatters["[%s]=%Q,"]
 local f_indexed_number  = formatters["[%s]=%s,"]
 local f_indexed_boolean = formatters["[%s]=%l,"]
 local f_indexed_table   = formatters["[%s]="]
 
-local f_ordered_string  = formatters["%q,"]
+local f_ordered_string  = formatters["%Q,"]
 local f_ordered_number  = formatters["%s,"]
 local f_ordered_boolean = formatters["%l,"]
 
-function table.fastserialize(t,prefix)
+function table.fastserialize(t,prefix) -- todo, move local function out
 
     -- prefix should contain the =
     -- not sorted
@@ -334,7 +336,6 @@ function table.fastserialize(t,prefix)
 
     local r = { type(prefix) == "string" and prefix or "return" }
     local m = 1
-
     local function fastserialize(t,outer) -- no mixes
         local n = #t
         m = m + 1
@@ -524,27 +525,27 @@ local f_start_key_nop     = formatters["%w{"]
 local f_stop              = formatters["%w},"]
 
 local f_key_num_value_num = formatters["%w[%s]=%s,"]
-local f_key_str_value_num = formatters["%w[%q]=%s,"]
+local f_key_str_value_num = formatters["%w[%Q]=%s,"]
 local f_key_boo_value_num = formatters["%w[%l]=%s,"]
 
-local f_key_num_value_str = formatters["%w[%s]=%q,"]
-local f_key_str_value_str = formatters["%w[%q]=%q,"]
-local f_key_boo_value_str = formatters["%w[%l]=%q,"]
+local f_key_num_value_str = formatters["%w[%s]=%Q,"]
+local f_key_str_value_str = formatters["%w[%Q]=%Q,"]
+local f_key_boo_value_str = formatters["%w[%l]=%Q,"]
 
 local f_key_num_value_boo = formatters["%w[%s]=%l,"]
-local f_key_str_value_boo = formatters["%w[%q]=%l,"]
+local f_key_str_value_boo = formatters["%w[%Q]=%l,"]
 local f_key_boo_value_boo = formatters["%w[%l]=%l,"]
 
 local f_key_num_value_not = formatters["%w[%s]={},"]
-local f_key_str_value_not = formatters["%w[%q]={},"]
+local f_key_str_value_not = formatters["%w[%Q]={},"]
 local f_key_boo_value_not = formatters["%w[%l]={},"]
 
 local f_key_num_value_seq = formatters["%w[%s]={ %, t },"]
-local f_key_str_value_seq = formatters["%w[%q]={ %, t },"]
+local f_key_str_value_seq = formatters["%w[%Q]={ %, t },"]
 local f_key_boo_value_seq = formatters["%w[%l]={ %, t },"]
 
 local f_val_num           = formatters["%w%s,"]
-local f_val_str           = formatters["%w%q,"]
+local f_val_str           = formatters["%w%Q,"]
 local f_val_boo           = formatters["%w%l,"]
 local f_val_not           = formatters["%w{},"]
 local f_val_seq           = formatters["%w{ %, t },"]
@@ -553,7 +554,7 @@ local f_fin_seq           = formatters[" %, t }"]
 local f_table_return      = formatters["return {"]
 local f_table_name        = formatters["%s={"]
 local f_table_direct      = formatters["{"]
-local f_table_entry       = formatters["[%q]={"]
+local f_table_entry       = formatters["[%Q]={"]
 local f_table_finish      = formatters["}"]
 
 ----- f_string            = formatters["%q"]
@@ -654,19 +655,20 @@ local function serialize(root,name,specification)
         -- we could check for k (index) being number (cardinal)
         if root and next(root) ~= nil then
             local first = nil
-            local last  = 0
-            last = #root
-            for k=1,last do
-                if rawget(root,k) == nil then
-             -- if root[k] == nil then
-                    last = k - 1
-                    break
+            local last  = #root
+            if last > 0 then
+                for k=1,last do
+                    if rawget(root,k) == nil then
+                 -- if root[k] == nil then
+                        last = k - 1
+                        break
+                    end
+                end
+                if last > 0 then
+                    first = 1
                 end
             end
-            if last > 0 then
-                first = 1
-            end
-            local sk = sortedkeys(root) -- inline fast version?\
+            local sk = sortedkeys(root)
             for i=1,#sk do
                 local k  = sk[i]
                 local v  = root[k]
@@ -820,3 +822,53 @@ if setinspector then
     end)
 end
 
+-- ordered hashes (for now here but in the table namespace):
+
+-- local t = table.orderedhash()
+--
+-- t["1"]  = { "a", "b" }
+-- t["2"]  = { }
+-- t["2a"] = { "a", "c", "d" }
+--
+-- for k, v in table.ordered(t) do
+--     ...
+-- end
+
+local mt = {
+    __newindex = function(t,k,v)
+        local n = t.last + 1
+        t.last    = n
+        t.list[n] = k
+        t.hash[k] = v
+    end,
+    __index = function(t,k)
+        return t.hash[k]
+    end,
+    __len = function(t)
+        return t.last
+    end,
+}
+
+function table.orderedhash()
+    return setmetatable({ list = { }, hash = { }, last = 0 }, mt)
+end
+
+function table.ordered(t)
+    local n = t.last
+    if n > 0 then
+        local l = t.list
+        local i = 1
+        local h = t.hash
+        local f = function()
+            if i <= n then
+                local k = i
+                local v = h[l[k]]
+                i = i + 1
+                return k, v
+            end
+        end
+        return f, 1, h[l[1]]
+    else
+        return function() end
+    end
+end
