@@ -49,6 +49,8 @@ constructors.cache           = containers.define("fonts", "constructors", constr
 constructors.privateoffset   = fonts.privateoffsets.textbase or 0xF0000
 constructors.cacheintex      = true -- so we see the original table in fonts.font
 
+constructors.addtounicode    = true
+
 -- This might become an interface:
 
 local designsizes           = allocate()
@@ -424,6 +426,8 @@ function constructors.scale(tfmdata,specification)
     targetparameters.forcedsize  = forcedsize  -- context specific
     targetparameters.extrafactor = extrafactor -- context specific
     --
+    local addtounicode  = constructors.addtounicode
+    --
     local tounicode     = fonts.mappings.tounicode
     local unknowncode   = tounicode(0xFFFD)
     --
@@ -543,7 +547,6 @@ function constructors.scale(tfmdata,specification)
     local hasitalics       = properties.hasitalics
     local autoitalicamount = properties.autoitalicamount
     local stackmath        = not properties.nostackmath
-    local nonames          = properties.noglyphnames
     local haskerns         = properties.haskerns     or properties.mode == "base" -- we can have afm in node mode
     local hasligatures     = properties.hasligatures or properties.mode == "base" -- we can have afm in node mode
     local realdimensions   = properties.realdimensions
@@ -592,7 +595,6 @@ function constructors.scale(tfmdata,specification)
         targetparameters.descender = delta * descender
     end
     --
--- inspect(targetparameters)
     constructors.enhanceparameters(targetparameters) -- official copies for us, now virtual
     --
     local protrusionfactor = (targetquad ~= 0 and 1000/targetquad) or 0
@@ -673,9 +675,10 @@ function constructors.scale(tfmdata,specification)
             description = descriptions[unicode] or character
             index       = description.index or unicode
         end
-        local width  = description.width
-        local height = description.height
-        local depth  = description.depth
+        local width     = description.width
+        local height    = description.height
+        local depth     = description.depth
+        local isunicode = description.unicode
         if realdimensions then
             -- this is mostly for checking issues
             if not height or height == 0 then
@@ -702,16 +705,16 @@ function constructors.scale(tfmdata,specification)
     --  if depth  then depth  = vdelta*depth  else depth  = scaleddepth  end
         if depth and depth ~= 0 then
             depth = delta*depth
-            if nonames then
+            if isunicode then
                 chr = {
-                    index  = index,
-                    height = height,
-                    depth  = depth,
-                    width  = width,
+                    index   = index,
+                    height  = height,
+                    depth   = depth,
+                    width   = width,
+                    unicode = isunicode,
                 }
             else
                 chr = {
-                    name   = description.name,
                     index  = index,
                     height = height,
                     depth  = depth,
@@ -719,30 +722,23 @@ function constructors.scale(tfmdata,specification)
                 }
             end
         else
-            -- this saves a little bit of memory time and memory, esp for big cjk fonts
-            if nonames then
+            if isunicode then
                 chr = {
-                    index  = index,
-                    height = height,
-                    width  = width,
+                    index   = index,
+                    height  = height,
+                    width   = width,
+                    unicode = isunicode,
                 }
             else
                 chr = {
-                    name   = description.name,
                     index  = index,
                     height = height,
                     width  = width,
                 }
             end
         end
-        local isunicode = description.unicode
-        if isunicode then
-            chr.unicode   = isunicode
-            chr.tounicode = tounicode(isunicode)
-            -- in luatex > 0.85 we can do this:
-            -- chr.tounicode = isunicode
-        else
-            chr.tounicode = unknowncode
+        if addtounicode then
+            chr.tounicode = isunicode and tounicode(isunicode) or unknowncode
         end
         if hasquality then
             -- we could move these calculations elsewhere (saves calculations)
@@ -817,7 +813,10 @@ function constructors.scale(tfmdata,specification)
             if stackmath then
                 local mk = character.mathkerns
                 if mk then
-                    local tr, tl, br, bl = mk.topright, mk.topleft, mk.bottomright, mk.bottomleft
+                    local tr = mk.topright
+                    local tl = mk.topleft
+                    local br = mk.bottomright
+                    local bl = mk.bottomleft
                     chr.mathkern = { -- singular -> should be patched in luatex !
                         top_right    = tr and mathkerns(tr,vdelta) or nil,
                         top_left     = tl and mathkerns(tl,vdelta) or nil,
@@ -983,15 +982,15 @@ function constructors.finalize(tfmdata)
     end
     --
     if not parameters.slantfactor then
-        parameters.slantfactor = tfmdata.slant or 0
+        parameters.slantfactor = (tfmdata.slant or 0)/1000
     end
     --
     if not parameters.extendfactor then
-        parameters.extendfactor = tfmdata.extend or 0
+        parameters.extendfactor = (tfmdata.extend or 1000)/1000
     end
     --
     if not parameters.squeezefactor then
-        parameters.squeezefactor = tfmdata.squeeze or 0
+        parameters.squeezefactor = (tfmdata.squeeze or 1000)/1000
     end
     --
     local designsize = parameters.designsize
@@ -1027,11 +1026,11 @@ function constructors.finalize(tfmdata)
         properties.virtualized = tfmdata.type == "virtual"
     end
     --
-    properties.fontname      = tfmdata.fontname
-    properties.filename      = tfmdata.filename
-    properties.fullname      = tfmdata.fullname
-    properties.name          = tfmdata.name
-    properties.psname        = tfmdata.psname
+    properties.fontname      = properties.fontname or tfmdata.fontname
+    properties.filename      = properties.filename or tfmdata.filename
+    properties.fullname      = properties.fullname or tfmdata.fullname
+    properties.name          = properties.name     or tfmdata.name
+    properties.psname        = properties.psname   or tfmdata.psname
     --
     properties.encodingbytes = tfmdata.encodingbytes or 1
     properties.embedding     = tfmdata.embedding     or "subset"
@@ -1164,7 +1163,9 @@ loose our testcases for <l n='luatex'/>.</p>
 --ldx]]--
 
 function constructors.hashinstance(specification,force)
-    local hash, size, fallbacks = specification.hash, specification.size, specification.fallbacks
+    local hash      = specification.hash
+    local size      = specification.size
+    local fallbacks = specification.fallbacks
     if force or not hash then
         hash = constructors.hashfeatures(specification)
         specification.hash = hash
@@ -1602,7 +1603,8 @@ end
 -- while typesetting
 
 function constructors.collectprocessors(what,tfmdata,features,trace,report)
-    local processes, nofprocesses = { }, 0
+    local processes    = { }
+    local nofprocesses = 0
     if features and next(features) then
         local properties     = tfmdata.properties
         local whathandler    = handlers[what]
