@@ -7,8 +7,8 @@
 
 local ProvidesLuaModule = { 
     name          = "luaotfload-configuration",
-    version       = "2.96",       --TAGVERSION
-    date          = "2019-02-14", --TAGDATE
+    version       = "3.00",       --TAGVERSION
+    date          = "2019-09-13", --TAGDATE
     description   = "luaotfload submodule / config file reader",
     license       = "GPL v2.0"
 }
@@ -1011,27 +1011,24 @@ end
 ---                                 EXPORTS
 -------------------------------------------------------------------------------
 
-return {
-  init = function ()
-    config.luaotfload = { }
-    logreport         = luaotfload.log.report
-    local parsers     = luaotfload.parsers
-    config_parser     = parsers.config
-    stripslashes      = parsers.stripslashes
+return function ()
+  config.luaotfload = { }
+  logreport         = luaotfload.log.report
+  local parsers     = luaotfload.parsers
+  config_parser     = parsers.config
+  stripslashes      = parsers.stripslashes
 
-    luaotfload.default_config = default_config
-    config.actions = {
-      read             = read,
-      apply            = apply,
-      apply_defaults   = apply_defaults,
-      reconfigure      = reconfigure,
-      dump             = dump,
-    }
-    if not apply_defaults () then
-      logreport ("log", 0, "load",
-                 "Configuration unsuccessful: error loading default settings.")
-    end
-    return true
+  luaotfload.default_config = default_config
+  config.actions = {
+    read             = read,
+    apply            = apply,
+    apply_defaults   = apply_defaults,
+    reconfigure      = reconfigure,
+    dump             = dump,
+  }
+  if not apply_defaults () then
+    logreport ("log", 0, "load",
+               "Configuration unsuccessful: error loading default settings.")
   end
-}
-
+  return true
+end
