@@ -7,8 +7,8 @@
 
 local ProvidesLuaModule = { 
     name          = "luaotfload-init",
-    version       = "2.96",       --TAGVERSION
-    date          = "2019-02-14", --TAGDATE
+    version       = "3.00",       --TAGVERSION
+    date          = "2019-09-13", --TAGDATE
     description   = "luaotfload submodule / initialization",
     license       = "GPL v2.0"
 }
@@ -78,44 +78,6 @@ local logreport  --- filled in after loading the log module
 
 --doc]]--
 
-local glyph_codes =
-  { [0] = "character"
-  , [1] = "glyph"
-  , [2] = "ligature"
-  , [3] = "ghost"
-  , [4] = "left"
-  , [5] = "right"
-  }
-
-local disc_codes  =
-  { [0] = "discretionary"
-  , [1] = "explicit"
-  , [2] = "automatic"
-  , [3] = "regular"
-  , [4] = "first"
-  , [5] = "second"
-  }
-
-local node_types = { disc = disc_codes, glyph = glyph_codes }
-
-local luatex_stubs = function ()
-  if not node.subtypes then
-    node.subtypes = function (t) return node_types [t] or { } end
-    local direct = node.direct
-
-    local getfield = direct.getfield
-    local setfield = direct.setfield
-
-    direct.setchar = direct.setchar or function (n, ...) setfield (n, "char", ...) end
-    direct.setprev = direct.setprev or function (n, ...) setfield (n, "prev", ...) end
-    direct.setnext = direct.setnext or function (n, ...) setfield (n, "next", ...) end
-
-    direct.getchar = direct.getchar or function (n) getfield (n, "char") end
-    direct.getprev = direct.getprev or function (n) getfield (n, "prev") end
-    direct.getnext = direct.getnext or function (n) getfield (n, "next") end
-  end
-end
-
 local init_early = function ()
 
   local store                  = { }
@@ -132,8 +94,6 @@ local init_early = function ()
   if not lualibs    then error "this module requires Luaotfload" end
   if not luaotfload then error "this module requires Luaotfload" end
 
-  luatex_stubs ()
-
   --[[doc--
 
     The logger needs to be in place prior to loading the fontloader due
@@ -285,7 +245,7 @@ local context_modules = {
   { ctx,   "font-onr"          },
   { ctx,   "font-one"          },
   { ctx,   "font-afk"          },
-  { ctx,   "font-tfm"          },
+  { ctx,   "luatex-fonts-tfm"  },
   { ctx,   "font-lua"          },
   { ctx,   "font-def"          },
   { ltx,   "luatex-fonts-def"  },  
@@ -502,6 +462,14 @@ end --- [init_adapt]
 
 local init_main = function ()
 
+  --[[doc--
+
+      First save a copy of font.each in order to also catch old-style fonts
+
+  --doc]]--
+
+  font.originaleach = font.each
+
   local load_fontloader_module = luaotfload.loaders.fontloader
   local ignore_module          = luaotfload.loaders.ignore
 
@@ -579,7 +547,7 @@ local init_main = function ()
     load_fontloader_module "font-onr"
     load_fontloader_module "font-one"
     load_fontloader_module "font-afk"
-    load_fontloader_module "font-tfm"
+    load_fontloader_module "fonts-tfm"
     load_fontloader_module "font-lua"
     load_fontloader_module "font-def"
     load_fontloader_module "fonts-def"
@@ -688,7 +656,11 @@ local init_post_install_callbacks = function ()
   -- MK Pass current text direction to simple_font_handler
   local handler = nodes.simple_font_handler
   local callback = function(head, groupcode, _, _, direction)
-    return handler(head, groupcode, nil, nil, direction or tex.get'textdir')
+    if not direction then
+      direction = tex.get'textdir'
+    end
+    domultiscript(head, nil, nil, nil, direction)
+    return handler(head, groupcode, nil, nil, direction)
   end
   luatexbase.add_to_callback("pre_linebreak_filter",
                              callback,
