@@ -8,8 +8,8 @@
 
 local ProvidesLuaModule = { 
     name          = "luaotfload-loaders",
-    version       = "2.95",       --TAGVERSION
-    date          = "2019-01-28", --TAGDATE
+    version       = "3.00",       --TAGVERSION
+    date          = "2019-09-13", --TAGDATE
     description   = "luaotfload submodule / callback handling",
     license       = "GPL v2.0"
 }
@@ -138,9 +138,9 @@ do
       --- We need to test for `encodingbytes` to avoid passing
       --- tfm fonts to `patch_font`. These fonts are fragile
       --- because they use traditional TeX font handling.
-      luatexbase.call_callback ("luaotfload.patch_font", fontdata, specification)
+      luatexbase.call_callback ("luaotfload.patch_font", fontdata, specification, id)
     else
-      luatexbase.call_callback ("luaotfload.patch_font_unsafe", fontdata, specification)
+      luatexbase.call_callback ("luaotfload.patch_font_unsafe", fontdata, specification, id)
     end
     return fontdata
   end
@@ -226,18 +226,15 @@ local install_callbacks = function ()
   return true
 end
 
-return {
-  init = function ()
-    local ret = true
-    if not install_formats () then
-      logreport ("log", 0, "loaders", "Error initializing OFM/PF{A,B} loaders.")
-      ret = false
-    end
-    if not install_callbacks () then
-      logreport ("log", 0, "loaders", "Error installing font loader callbacks.")
-      ret = false
-    end
-    return ret
+return function ()
+  if not install_formats () then
+    logreport ("log", 0, "loaders", "Error initializing OFM/PF{A,B} loaders.")
+    return false
   end
-}
+  if not install_callbacks () then
+    logreport ("log", 0, "loaders", "Error installing font loader callbacks.")
+    return false
+  end
+  return true
+end
 -- vim:tw=79:sw=2:ts=2:expandtab
