@@ -1,4 +1,4 @@
-# $Id: TLCrypto.pm 50692 2019-04-01 15:01:10Z preining $
+# $Id: TLCrypto.pm 52158 2019-09-23 18:04:33Z karl $
 # TeXLive::TLCrypto.pm - handle checksums and signatures.
 # Copyright 2016-2019 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
@@ -12,7 +12,7 @@ use TeXLive::TLConfig;
 use TeXLive::TLUtils qw(debug ddebug win32 which platform
                         conv_to_w32_path tlwarn tldie);
 
-my $svnrev = '$Revision: 50692 $';
+my $svnrev = '$Revision: 52158 $';
 my $_modulerevision = ($svnrev =~ m/: ([0-9]+) /) ? $1 : "unknown";
 sub module_revision { return $_modulerevision; }
 
@@ -148,7 +148,9 @@ sub tlchecksum {
   if (!$::checksum_method) {
     setup_checksum_method();
   }
-  tldie("no checksum method available\n") if (!$::checksum_method);
+  tldie("TLCRYPTO::tlchecksum: no checksum method available\n")
+    if (!$::checksum_method);
+
   if (-r $file) {
     my ($out, $ret);
     if ($::checksum_method eq "openssl") {
@@ -167,10 +169,10 @@ sub tlchecksum {
       close(FILE);
       $ret = 0;
     } else {
-      tldie("unknown checksum program: $::checksum_method\n");
+      tldie("TLCRYPTO::tlchecksum: unknown checksum program: $::checksum_method\n");
     }
     if ($ret != 0) {
-      tlwarn("tlchecksum: cannot compute checksum: $file\n");
+      tlwarn("TLCRYPTO::tlchecksum: cannot compute checksum: $file\n");
       return "";
     }
     ddebug("tlchecksum: out = $out\n");
@@ -186,12 +188,13 @@ sub tlchecksum {
     }
     debug("tlchecksum($file): ===$cs===\n");
     if (length($cs) != 128) {
-      tlwarn("unexpected output from $::checksum_method: $out\n");
+      tlwarn("TLCRYPTO::tlchecksum: unexpected output from $::checksum_method:"
+             . " $out\n");
       return "";
     }
     return $cs;
   } else {
-    tlwarn("tlchecksum: given file not readable: $file\n");
+    tlwarn("TLCRYPTO::tlchecksum: given file not readable: $file\n");
     return "";
   }
 }
@@ -472,9 +475,11 @@ cryptographic signature verification of
   $file
 against
   $signature_url
-failed. Output was
+failed. Output was:
 $out
-Please report to texlive\@tug.org
+Please try from a different mirror and/or wait a few minutes
+and try again; usually this is because of transient updates.
+If problems persist, feel free to report to texlive\@tug.org.
 GPGERROR
       }
     } else {
