@@ -1,12 +1,12 @@
-# $Id: TLPOBJ.pm 48521 2018-08-31 03:41:21Z preining $
+# $Id: TLPOBJ.pm 50211 2019-03-03 21:52:59Z preining $
 # TeXLive::TLPOBJ.pm - module for using tlpobj files
-# Copyright 2007-2018 Norbert Preining
+# Copyright 2007-2019 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
 
 package TeXLive::TLPOBJ;
 
-my $svnrev = '$Revision: 48521 $';
+my $svnrev = '$Revision: 50211 $';
 my $_modulerevision = ($svnrev =~ m/: ([0-9]+) /) ? $1 : "unknown";
 sub module_revision { return $_modulerevision; }
 
@@ -119,16 +119,17 @@ sub from_fh {
           # do manual parsing
           # this is not optimal, but since we support only two tags there
           # are not so many cases
-          if ($rest =~ m/^details="(.*)"\s*$/) {
-            $self->{'docfiledata'}{$f}{'details'} = $1;
-          } elsif ($rest =~ m/^language="(.*)"\s*$/) {
-            $self->{'docfiledata'}{$f}{'language'} = $1;
-          } elsif ($rest =~ m/^language="(.*)"\s+details="(.*)"\s*$/) {
+          # Warning: need tp check the double cases first!!!
+          if ($rest =~ m/^language="(.*)"\s+details="(.*)"\s*$/) {
             $self->{'docfiledata'}{$f}{'details'} = $2;
             $self->{'docfiledata'}{$f}{'language'} = $1;
           } elsif ($rest =~ m/^details="(.*)"\s+language="(.*)"\s*$/) {
             $self->{'docfiledata'}{$f}{'details'} = $1;
             $self->{'docfiledata'}{$f}{'language'} = $2;
+          } elsif ($rest =~ m/^details="(.*)"\s*$/) {
+            $self->{'docfiledata'}{$f}{'details'} = $1;
+          } elsif ($rest =~ m/^language="(.*)"\s*$/) {
+            $self->{'docfiledata'}{$f}{'language'} = $1;
           } else {
             tlwarn("$0: Unparsable tagging in TLPDB line: $line\n");
           }
@@ -735,6 +736,13 @@ sub make_container {
         tlwarn("$0: Couldn't compress $destdir/$tarname\n");
         return (0,0, "");
       }
+      # make sure we remove the original tar since old lz4 versions
+      # cannot automatically delete it.
+      # We remove the tar file only when the compressed file was
+      # correctly created, something that should only happen in the
+      # most strange cases.
+      unlink("$destdir/$tarname")
+        if ((-r "$destdir/$tarname") && (-r "$destdir/$containername"));
     } else {
       tlwarn("$0: Couldn't find $destdir/$tarname to run $compressor\n");
       return (0, 0, "");
@@ -822,7 +830,7 @@ sub update_from_catalogue {
       $foo =~ s/^.Date: //;
       # trying to extract the interesting part of a subversion date
       # keyword expansion here, e.g.,
-      # $Date: 2018-08-31 05:41:21 +0200 (Fri, 31 Aug 2018) $
+      # $Date: 2019-03-03 22:52:59 +0100 (Sun, 03 Mar 2019) $
       # ->2007-08-15 19:43:35 +0100
       $foo =~ s/ \(.*\)( *\$ *)$//;  # maybe nothing after parens
       $self->cataloguedata->{'date'} = $foo;
