@@ -1,5 +1,11 @@
 $mirrors = {
   'Africa' => {
+    'Algeria' => {
+      'http://ctan.epst-tlemcen.dz/tex-archive/' => 1
+    },
+    'Morocco' => {
+      'https://mirror.marwan.ma/ctan/' => 1
+    },
     'South Africa' => {
       'http://ftp.leg.uct.ac.za/pub/packages/ctan/' => 1,
       'http://ftp.sun.ac.za/ftp/CTAN/' => 1,
@@ -9,19 +15,24 @@ $mirrors = {
   'Asia' => {
     'China' => {
       'http://mirror.lzu.edu.cn/CTAN/' => 1,
+      'http://mirrors.cqu.edu.cn/CTAN/' => 1,
       'http://mirrors.geekpie.club/CTAN/' => 1,
-      'http://mirrors.huaweicloud.com/repository/toolkit/CTAN/' => 1,
       'http://mirrors.sjtug.sjtu.edu.cn/ctan/' => 1,
       'http://mirrors.tuna.tsinghua.edu.cn/CTAN/' => 1,
       'http://mirrors.ustc.edu.cn/CTAN/' => 1
     },
+    'Hong Kong' => {
+      'https://mirror-hk.koddos.net/CTAN/' => 1
+    },
     'India' => {
       'http://ctan.imsc.res.in/' => 1,
       'http://mirror.iopb.res.in/tex-archive/' => 1
     },
+    'Indonesia' => {
+      'https://mirror.unpad.ac.id/ctan/' => 1
+    },
     'Iran' => {
       'http://ctan.asis.ai/' => 1,
-      'http://ctan.um.ac.ir/' => 1,
       'http://ctan.yazd.ac.ir/' => 1
     },
     'Japan' => {
@@ -34,11 +45,15 @@ $mirrors = {
       'http://ftp.harukasan.org/CTAN/' => 1,
       'http://ftp.kaist.ac.kr/pub/tex-archive/' => 1,
       'http://ftp.ktug.org/tex-archive/' => 1,
-      'http://mirror.navercorp.com/CTAN/' => 1
+      'http://mirror.navercorp.com/CTAN/' => 1,
+      'http://mirror.yongbok.net/ctan/' => 1
     },
     'Philippines' => {
       'http://mirror.pregi.net/tex-archive/' => 1
     },
+    'Singapore' => {
+      'https://download.nus.edu.sg/mirror/ctan/' => 1
+    },
     'Taiwan' => {
       'ftp://ftp.ccu.edu.tw/pub/tex/' => 1,
       'http://ftp.yzu.edu.tw/CTAN/' => 1
@@ -46,7 +61,8 @@ $mirrors = {
   },
   'Europe' => {
     'Austria' => {
-      'http://mirror.easyname.at/ctan/' => 1
+      'http://mirror.easyname.at/ctan/' => 1,
+      'http://mirror.kumi.systems/ctan/' => 1
     },
     'Belarus' => {
       'http://mirror.datacenter.by/pub/mirrors/CTAN/' => 1
@@ -58,7 +74,8 @@ $mirrors = {
       'http://mirrors.dotsrc.org/ctan/' => 1
     },
     'Finland' => {
-      'ftp://ftp.funet.fi/pub/TeX/CTAN/' => 1
+      'ftp://ftp.funet.fi/pub/TeX/CTAN/' => 1,
+      'https://www.texlive.info/CTAN/' => 1
     },
     'France' => {
       'http://ctan.crest.fr/tex-archive/' => 1,
@@ -67,22 +84,29 @@ $mirrors = {
       'http://distrib-coffee.ipsl.jussieu.fr/pub/mirrors/ctan/' => 1,
       'http://mirror.ibcp.fr/pub/CTAN/' => 1,
       'http://mirrors.ircam.fr/pub/CTAN/' => 1,
-      'http://mirrors.standaloneinstaller.com/ctan/' => 1
+      'http://mirrors.standaloneinstaller.com/ctan/' => 1,
+      'https://mirrors.chevalier.io/CTAN/' => 1
     },
     'Germany' => {
       'ftp://ftp.fu-berlin.de/tex/CTAN/' => 1,
       'ftp://ftp.rrzn.uni-hannover.de/pub/mirror/tex-archive/' => 1,
       'ftp://ftp.tu-chemnitz.de/pub/tex/' => 1,
+      'http://ctan.ebinger.cc/tex-archive/' => 1,
       'http://ctan.mirror.norbert-ruehl.de/' => 1,
       'http://ctan.space-pro.be/tex-archive/' => 1,
       'http://ftp.fau.de/ctan/' => 1,
       'http://mirror.informatik.hs-fulda.de/tex-archive/' => 1,
       'http://packages.oth-regensburg.de/ctan/' => 1,
-      'http://vesta.informatik.rwth-aachen.de/ftp/pub/mirror/ctan/' => 1
+      'http://vesta.informatik.rwth-aachen.de/ftp/pub/mirror/ctan/' => 1,
+      'https://ctan.kako-dev.de/' => 1,
+      'https://ctan.net/' => 1
     },
     'Greece' => {
       'http://ftp.cc.uoc.gr/mirrors/CTAN/' => 1
     },
+    'Hungary' => {
+      'https://mirror.szerverem.hu/ctan/' => 1
+    },
     'Italy' => {
       'http://ctan.mirror.garr.it/mirrors/CTAN/' => 1
     },
@@ -102,8 +126,8 @@ $mirrors = {
     },
     'Portugal' => {
       'ftp://ftp.di.uminho.pt/pub/ctan/' => 1,
-      'ftp://ftp.ist.utl.pt/pub/ctan/' => 1,
-      'http://mirrors.up.pt/pub/CTAN/' => 1
+      'http://mirrors.up.pt/pub/CTAN/' => 1,
+      'https://ftp.eq.uc.pt/software/TeX/' => 1
     },
     'Romania' => {
       'http://mirrors.nxthost.com/ctan/' => 1
