@@ -44,7 +44,7 @@ catch {rename send {}}
 set ::instroot [file normalize [info script]]
 set ::instroot [file dirname [file dirname [file dirname $::instroot]]]
 
-# declarations and procs shared with tlshell.tcl
+# declarations, initializations and procs shared with tlshell.tcl
 source [file join $::instroot "tlpkg" "tltcl" "tltcl.tcl"]
 
 ### initialize some globals ###
@@ -56,19 +56,44 @@ set ::out_log {}; # list of strings
 
 set ::perlbin "perl"
 if {$::tcl_platform(platform) eq "windows"} {
-  set ::perlbin "${::instroot}/tlpkg/tlperl/bin/perl.exe"
+  # the batch wrapper should have put perl.exe on the searchpath
+  set ::perlbin "perl.exe"
 }
 
 # menu modes
 set ::advanced 0
 set ::alltrees 0
 
+# interactively select repository
+set ::select_repo 0
+
+proc kill_perl {} {
+  if $::perlpid {
+    catch {
+      if {$::tcl_platform(platform) eq "unix"} {
+        exec -ignorestderr kill $::perlpid >& /dev/null
+      } else {
+        exec -ignorestderr taskkill /pid $::perlpid /t /f
+      }
+    }
+  }
+}
+
+proc err_exit {{mess ""}} {
+  if {$mess eq ""} {set mess "Error"}
+  append mess "\n" [get_stacktrace]
+  tk_messageBox -icon error -message $mess
+  # kill perl process, just in case
+  kill_perl
+  exit
+} ; # err_exit
+
 # warning about non-empty target tree
 set ::td_warned 0
 
 proc is_nonempty {td} {
   if {! [file exists $td]} {return 0}
-  return [expr {[llength [glob -directory $td *]] > 0}]
+  return [expr {[llength [glob -nocomplain -directory $td *]] > 0}]
 }
 
 proc td_warn {td} {
@@ -101,19 +126,23 @@ proc show_time {s} {
 # write to a logfile which is shared with the backend.
 # both parties open, append and close every time.
 
-#if {$::tcl_platform(os) eq "Darwin"} {
-#  set ::dblfile "$::env(TMPDIR)/dblog"
-#} elseif {$::tcl_platform(platform) eq "unix"} {
-#  set ::dblfile "/tmp/dblog"
-#} else {
-#  set ::dblfile "$::env(TEMP)/dblog.txt"
-#}
-#proc dblog {s} {
-#  set db [open $::dblfile a]
-#  set t [get_stacktrace]
-#  puts $db "TCL: $s\n$t"
-#  close $db
-#}
+set dblogdir "/tmp"
+if [info exists ::env(TMPDIR)] {
+  set dblogdir $::env(TMPDIR)
+} elseif [info exists ::env(TMP)] {
+  set dblogdir $::env(TMP)
+} elseif [info exists ::env(TEMP)] {
+  set dblogdir $::env(TEMP)
+}
+set ::dblfile [file join $dblogdir "dblog"]
+unset dblogdir
+
+proc dblog {s} {
+  set db [open $::dblfile a]
+  set t [get_stacktrace]
+  puts $db "TCL: $s\n$t"
+  close $db
+}
 
 proc maybe_print_welcome {} {
   # if the last non-empty line was "All done", then installation is completed.
@@ -124,7 +153,6 @@ proc maybe_print_welcome {} {
   for {set i [.log.tx count -lines 1.0 end]} {$i > 0} {incr i -1} {
     set l  [.log.tx get ${i}.0 ${i}.end]
     if {$l ne ""} {
-      puts $l
       if {[string range $l 0 11] eq "Installed on"} {
         set all_done 1
       }
@@ -195,13 +223,30 @@ This should not have happened!"
   return $l
 }; # read_line_no_eof
 
+# non-blocking i/o: callback for "readable" while the splash screen is up
+# and the back end tries to contact the repository
+proc read_line_loading {} {
+  set l "" ; # will contain the line to be read
+  if {([catch {chan gets $::inst l} len] || [chan eof $::inst])} {
+    catch {chan close $::inst}
+    # note. the normal way to terminate is terminating the GUI shell.
+    # This closes stdin of the child
+  } elseif {$len >= 0} {
+    if {$l eq "endload"} {
+      chan configure $::inst -blocking 1
+      chan event $::inst readable {}
+      set ::loaded 1
+    }
+  }
+}; # read_line_loading
+
 # non-blocking i/o: callback for "readable" during stage 3, installation
 # ::out_log should no longer be needed
 proc read_line_cb {} {
   set l "" ; # will contain the line to be read
   if {([catch {chan gets $::inst l} len] || [chan eof $::inst])} {
     catch {chan close $::inst}
-    # note. the right way to terminate is terminating the GUI shell.
+    # note. the normal way to terminate is terminating the GUI shell.
     # This closes stdin of the child
     .close state !disabled
     if [winfo exists .abort] {.abort state disabled}
@@ -215,29 +260,161 @@ proc read_line_cb {} {
   }
 }; # read_line_cb
 
+proc maybe_abort {} {
+  set ans [tk_messageBox -message [__ "Really abort?"] -type yesno \
+               -default no]
+  if {$ans eq "no"} {
+    return
+  }
+  catch {chan close $::inst}
+  kill_perl
+  exit
+}
+
+# modify parameter list for either restarting gui installer
+# or for starting back end
+
+proc replace_lang_parameter {} {
+  # edit original command line by removing any language parameter
+  # and adding a repository parameter $m. same for language
+  set i [llength $::argv]
+  while {$i > 0} {
+    incr i -1
+    set p [lindex $::argv $i]
+    if {$p eq "-lang" || $p eq "-gui-lang"} {
+      set j [expr {$i+1}]
+      if {$j < [llength $::argv]} {
+        set ::argv [lreplace $::argv $i $j]
+      } else {
+        set ::argv [lreplace $::argv $i]
+      }
+      set ::argv [lreplace $::argv $i [expr {$i+1}]]
+    }
+  }
+  lappend ::argv "-lang" $::lang
+}
+
+proc replace_repo_parameter {m} {
+  # edit original command line by removing any repository parameter
+  # and adding a repository parameter $m. same for language
+  set i $::argc
+  while {$i > 0} {
+    incr i -1
+    set p [lindex $::argv $i]
+    if {$p in [list "-location" "-url" "-repository" "-repos" "-repo"]} {
+      set j [expr {$i+1}]
+      if {$j < [llength $::argv]} {
+        set ::argv [lreplace $::argv $i $j]
+      } else {
+        set ::argv [lreplace $::argv $i]
+      }
+    }
+  }
+  lappend ::argv "-repository" $m
+}
+
+# restart installer with chosen repository
+proc restart_with_mir {m} {
+  # edit original command line by removing any repository parameter
+  # and adding a repository parameter $m. same for language
+  replace_repo_parameter $m
+  replace_lang_parameter
+  set cmd [linsert $::argv 0 [info nameofexecutable] [info script] "--"]
+
+  # terminate back end
+  catch {chan close $::inst}
+  kill_perl
+
+  # restart install-tl with edited command-line
+  exec {*}$cmd &
+  exit
+} ; # restart_with_mir
+
+proc continue_with_mir {m} {
+  replace_repo_parameter $m
+  set ::mir_selected 1 ; # this will cause select_mirror to finish up
+}
+
+# add $::instroot as local repository if applicable
+proc mirror_menu_plus {wnd cmd} {
+  set have_local 0
+  if [file isdirectory [file join $::instroot "archive"]] {
+    set have_local 1
+  } elseif [file readable \
+                [file join $::instroot "texmf-dist" "web2c" "texmf.cnf"]] {
+    set have_local 1
+  }
+  mirror_menu $wnd $cmd
+  if {[winfo class $wnd] ne "TMenubutton"} {
+    error_exit "No mirror list found"
+  } else {
+    if $have_local {
+      $wnd.m insert 0 command -label "$::instroot ([__ "Local repository"])" \
+          -command "$cmd $::instroot"
+    }
+  }
+  return $wnd
+}
+
 ##############################################################
 
-##### special-purpose uses of main window: splash, log #####
+##### special-purpose uses of main window: select_mirror, splash, log #####
 
-proc make_splash {} {
-
-  # wm overrideredirect . true
+proc pre_splash {} {
+  # build splash window minus buttons
+  wm withdraw .
 
   # picture and logo
   catch {
     image create photo tlimage -file \
         [file join $::instroot "tlpkg" "installer" "texlion.gif"]
-    pack [frame .white -background white] -fill x -expand 1
+    pack [frame .white -background white] -side top -fill x -expand 1
     label .image -image tlimage -background white
     pack .image -in .white
   }
 
-  # wallpaper
+  # wallpaper for remaining widgets
   pack [ttk::frame .bg -padding 3] -fill both -expand 1
 
+  # buttons: abort button, mirrors dropdown menu
+  pack [ttk::frame .splfb] -in .bg -side bottom -fill x
+}
+
+proc select_mirror {} {
+
+  # buttons: abort button, mirrors dropdown menu, continue
+  ppack [mirror_menu_plus .splfb.slmir_m continue_with_mir] \
+      -side right
+  ppack [ttk::button .splfb.slmir_a -text [__ "Abort"] -command maybe_abort] \
+      -side right
+
+  wm attributes . -topmost
+  update
+  wm state . normal
+  raise .
+  vwait ::mir_selected
+} ; # select_mirror
+
+proc make_splash {} {
+
+  foreach c [winfo children .splfb] {
+    catch {destroy $c}
+  }
+  update
+
+  ppack [ttk::button .spl_a -text [__ "Abort"] -command maybe_abort] \
+      -side right -in .splfb
+  ppack [mirror_menu_plus .spl_o restart_with_mir] -side right -in .splfb
+
+  # some text
   ppack [ttk::label .text -text [__ "TeX Live Installer"] \
              -font bigfont] -in .bg
-  ppack [ttk::label .loading -text [__ "Loading..."]] -in .bg
+  if {! $::select_repo} {
+    ppack [ttk::label .loading -text [__ "Trying to load %s.
+
+If this takes too long, press Abort or choose another repository." \
+                                          $::prelocation]] -in .bg
+  }
 
   wm attributes . -topmost
   update
@@ -249,7 +426,7 @@ proc make_splash {} {
 proc show_log {{do_abort 0}} {
   wm withdraw .
   foreach c [winfo children .] {
-    destroy $c
+    catch {destroy $c}
   }
 
   # wallpaper
@@ -260,16 +437,7 @@ proc show_log {{do_abort 0}} {
   ttk::button .close -text [__ "Close"] -command exit
   ppack .close -in .bottom -side right
   if $do_abort {
-    ttk::button .abort -text [__ "Cancel"]  -command {
-      set ans [tk_messageBox -message [__ "Really abort?"] -type yesno \
-                   -default no]
-      if {$ans eq "no"} return
-      catch {chan close $::inst}
-      if {$::tcl_platform(platform) eq "windows"} {
-        catch {exec  taskkill /pid $::perlpid /t /f}
-      }
-      exit
-    }
+    ttk::button .abort -text [__ "Abort"]  -command maybe_abort
     ppack .abort -in .bottom -side right
   }
   bind . <Escape> {
@@ -354,7 +522,8 @@ proc edit_name {} {
   # widgets
   ttk::label .tled.l -text [__ "Change name (slashes not allowed)"]
   pack .tled.l -in .tled.bg -padx 5 -pady 5
-  ttk::entry .tled.e -width 20 -state normal
+  ttk::entry .tled.e -width 20
+  .tled.e state !disabled
   pack .tled.e -in .tled.bg -pady 5
   .tled.e insert 0 [.tltd.name_l cget -text]
 
@@ -370,11 +539,12 @@ proc edit_name {} {
     }
   }
   ppack .tled.ok_b -in .tled.buttons -side right -padx 5 -pady 5
-  ttk::button .tled.q_b -text [__ "Cancel"] -command {end_dlg "" .tled}
-  ppack .tled.q_b -in .tled.buttons -side right -padx 5 -pady 5
-  bind .tled <Escape> {.tled.q_b invoke}
+  ttk::button .tled.cancel_b -text [__ "Cancel"] -command {end_dlg "" .tled}
+  ppack .tled.cancel_b -in .tled.buttons -side right -padx 5 -pady 5
+  bind .tled <Escape> {.tled.cancel_b invoke}
 
-  wm protocol .tled WM_DELETE_WINDOW {.tled.q_b invoke}
+  wm protocol .tled WM_DELETE_WINDOW \
+      {cancel_or_destroy .tled.cancel_b .tled}
   wm resizable .tled 0 0
   place_dlg .tled .tltd
 } ; # edit_name
@@ -534,7 +704,8 @@ proc texdir_setup {} {
   bind .tltd <Return> commit_root
   bind .tltd <Escape> {destroy .tltd}
 
-  wm protocol .tltd  WM_DELETE_WINDOW {.tltd.cancel_b invoke}
+  wm protocol .tltd  WM_DELETE_WINDOW \
+      {cancel_or_destroy .tltd.cancel_b .tltd}
   wm resizable .tltd 1 0
   place_dlg .tltd
 } ; # texdir_setup
@@ -575,11 +746,12 @@ proc edit_dir {d} {
   ppack .td.cancel -in .td.f -side right
   bind .td <Escape> {.td.cancel invoke}
 
-  wm protocol .td WM_DELETE_WINDOW {.td.cancel invoke}
+  wm protocol .td WM_DELETE_WINDOW \
+      {cancel_or_destroy .td.cancel .td}
   wm resizable .td 1 0
   place_dlg .td .
   tkwait window .td
-  if {$::dialog_ans ne ""} {
+  if {[info exists ::dialog_ans] && $::dialog_ans ne ""} {
     set ::vars($d) [forward_slashify $::dialog_ans]
     if $::vars(instopt_portable) {
       if {$d eq "TEXMFLOCAL"} {set ::vars(TEXMFHOME) $::vars($d)}
@@ -792,7 +964,8 @@ proc select_binaries {} {
   bind .tlbin.lst <ButtonRelease-1> \
       {toggle_bin [.tlbin.lst identify item %x %y]}
 
-  wm protocol .tlbin WM_DELETE_WINDOW {.tlbin.cancel invoke}
+  wm protocol .tlbin WM_DELETE_WINDOW \
+      {cancel_or_destroy .tlbin.cancel .tlbin}
   wm resizable .tlbin 1 1
   place_dlg .tlbin .
 }; # select_binaries
@@ -848,7 +1021,8 @@ proc select_scheme {} {
   # we already made sure that $::vars(selected_scheme) has a valid value
   .tlschm.lst selection set [list $::vars(selected_scheme)]
 
-  wm protocol .tlschm WM_DELETE_WINDOW {tlschm.cancel invoke}
+  wm protocol .tlschm WM_DELETE_WINDOW \
+      {cancel_or_destroy tlschm.cancel .tlschm}
   wm resizable .tlschm 1 0
   place_dlg .tlschm .
 }; # select_scheme
@@ -969,7 +1143,8 @@ proc select_collections {} {
         [list [mark_sym $::vars($c)] [__ $::coll_descs($c)]]
   }
 
-  wm protocol .tlcoll WM_DELETE_WINDOW {.tlcoll.cancel invoke}
+  wm protocol .tlcoll WM_DELETE_WINDOW \
+      {cancel_or_destroy .tlcoll.cancel .tlcoll}
   wm resizable .tlcoll 1 1
   place_dlg .tlcoll .
 }; # select_collections
@@ -1130,11 +1305,12 @@ if {$::tcl_platform(platform) ne "windows"} {
 
     check_sym_entries
 
-    wm protocol .edsyms  WM_DELETE_WINDOW {.edsyms.cancel invoke}
+    wm protocol .edsyms  WM_DELETE_WINDOW \
+        {cancel_or_destroy .edsyms.cancel .edsyms}
     wm resizable .edsyms 1 0
     place_dlg .edsyms .
-  }
-}
+  } ; # edit_symlinks
+} ; # $::tcl_platform(platform) ne "windows"
 
 #############################################################
 
@@ -1149,12 +1325,12 @@ if {$::tcl_platform(platform) ne "windows"} {
 
 proc run_menu {} {
   if [info exists ::env(dbgui)] {
-    puts "\ndbgui: run_menu: advanced is now $::advanced"
-    puts "dbgui: run_menu: alltrees is now $::alltrees"
+    #puts "\ndbgui: run_menu: advanced is now $::advanced"
+    #puts "dbgui: run_menu: alltrees is now $::alltrees"
   }
   wm withdraw .
   foreach c [winfo children .] {
-    destroy $c
+    catch {destroy $c}
   }
 
   # wallpaper
@@ -1181,13 +1357,14 @@ proc run_menu {} {
   if {!$::advanced} {
     ppack [ttk::button .adv -text [__ "Advanced"] -command {
       set ::menu_ans "advanced"
-      if [info exists ::env(dbgui)] {puts "dbgui: requested advanced"}
+      #if [info exists ::env(dbgui)] {puts "dbgui: requested advanced"}
     }] -in .final -side left
   }
   pack [ttk::separator .seph1 -orient horizontal] \
       -in .bg -side bottom -pady 3 -fill x
 
   # directories, selections
+  # advanced and basic have different frame setups
   if $::advanced {
     pack [ttk::frame .left] -in .bg -side left -fill both -expand 1
     set curf .left
@@ -1196,8 +1373,6 @@ proc run_menu {} {
     set curf .main
   }
 
-  # labelframes do not look quite right on macos
-
   # directory section
   pack [ttk::frame .dirf] -in $curf -fill x
   grid columnconfigure .dirf 1 -weight 1
@@ -1205,6 +1380,8 @@ proc run_menu {} {
 
   if $::advanced {
     incr rw
+    # labelframes do not look quite right on macos,
+    # instead, separate label widget for title
     pgrid [ttk::label .dirftitle -text [__ "Installation root"] \
                -font hfont] \
         -in .dirf -row $rw -column 0 -columnspan 3 -sticky w
@@ -1288,7 +1465,7 @@ proc run_menu {} {
     if {!$::alltrees} {
       ttk::button .tmoreb -text [__ "More ..."] -command {
         set ::menu_ans "alltrees"
-        if [info exists ::env(dbgui)] {puts "dbgui: requested alltrees"}
+        #if [info exists ::env(dbgui)] {puts "dbgui: requested alltrees"}
       }
       pgrid .tmoreb -in .dirf -row $rw -column 2 -sticky ne
     }
@@ -1363,7 +1540,7 @@ proc run_menu {} {
   incr rw
   ttk::label .lsize -text [__ "Disk space required (in MB):"]
   ttk::label .size_req -textvariable ::vars(total_size)
-  pgrid .lsize -in $curf -row $rw -column 0 -sticky e
+  pgrid .lsize -in $curf -row $rw -column 0 -sticky w
   pgrid .size_req -in $curf -row $rw -column 1 -sticky w
 
   ########################################################
@@ -1536,10 +1713,10 @@ proc run_menu {} {
   if {[is_nonempty $::vars(TEXDIR)] && ! $::td_warned} {
     td_warn $::vars(TEXDIR)
   }
-  if [info exists ::env(dbgui)] {puts "dbgui: unsetting menu_ans"}
+  #if [info exists ::env(dbgui)] {puts "dbgui: unsetting menu_ans"}
   unset -nocomplain ::menu_ans
   vwait ::menu_ans
-  if [info exists ::env(dbgui)] {puts "dbgui0: menu_ans is $::menu_ans"}
+  #if [info exists ::env(dbgui)] {puts "dbgui0: menu_ans is $::menu_ans"}
   return $::menu_ans
 }; # run_menu
 
@@ -1733,84 +1910,139 @@ proc main_prog {} {
   wm title . [__ "TeX Live Installer"]
   wm protocol . WM_DELETE_WINDOW whataboutclose
 
-  # check for profile argument whether to put up a splash screen
+  if {[file exists $::dblfile]} {file delete $::dblfile}
+
+  # handle some command-line arguments.
+  # the argument list should already be normalized: '--' => '-', "=" => ' '
   set i 0
   set do_splash 1
-  while {$i < $::argc} {
+  set ::prelocation "..."
+  set ::mir_selected 1 ; # i.e. default or set by parameter
+  set i [llength $::argv]
+  while {$i > 0} {
+    incr i -1
     set p [lindex $::argv $i]
-    incr i
-    if {[string range $p 0 7] eq "-profile" || \
-            [string range $p 0 8] eq "--profile"} {
+    if {$p eq "-profile"} {
+      # check for profile argument: no splash screen if present
       set do_splash 0
-      break
+    } elseif {$p in [list "-location" "-url" "-repository" "-repos" "-repo"]} {
+      # check for repository argument: bail out if obviously invalid
+      set j [expr {$i+1}]
+      if {$j<[llength $::argv]} {
+        set p [lindex $::argv $j]
+        unset j
+        if {$p ne "ctan" && ! [possible_repository $p]} {
+          tk_messageBox -message [__ "%s not a local or remote repository" $p] \
+              -title [__ "Error"] -type ok -icon error
+          exit
+        }
+        set ::prelocation $p
+      } else {
+        tk_messageBox -message [__ "%s requires an argument" $p] \
+            -title [__ "Error"] -type ok -icon error
+        exit
+      }
+    } elseif {$p eq "-select-repository"} {
+      # in this case, we start with selecting a repository
+      # from a mirror list and modify ::argv to take the selection
+      # into account before contacting the perl back end.
+      unset -nocomplain ::mir_selected
+      # remove this argument
+      set ::argv [lreplace $::argv $i $i]
     }
   }
   unset i
+
+  if {$do_splash || ! [info exists ::mir_selected]} pre_splash
+
+  if {! [info exists ::mir_selected]} {
+    select_mirror
+  }
+
   if $do_splash make_splash
   unset do_splash
 
-  # start install-tl-[tcl] via a pipe.
-  # the command must be a string, not a list.
-  # therefore, arguments with spaces must be quoted.
-  # although we build the command at first as a list,
-  # it will be joined into a string at a second step
-  set cmd [list ${::perlbin} "${::instroot}/install-tl" \
-               "-from_ext_gui" {*}$::argv]
-  set i 0
-  while {$i<[llength $cmd]} {
-    set c [lindex $cmd $i]
-    if {[string first " " $c] >= 0} {
-      lset cmd $i "\"$c\""
-    }
-    incr i
+  if {! [info exist ::mir_selected]} {
+    vwait ::mir_selected
   }
-  unset i
-  # tk_messageBox -message [join $cmd " "] -title "debugging"
+
+  # start install-tl-[tcl] via a pipe.
+  set cmd [list "|${::perlbin}" "${::instroot}/install-tl" \
+               "-from_ext_gui" {*}$::argv 2>@1]
   show_time "opening pipe"
-  if [catch {open "|[join $cmd " "] 2>@1" r+} ::inst] {
-    # "2>@1" ok under Windows >= XP
+  if [catch {open $cmd r+} ::inst] {
     err_exit "Error starting Perl backend"
   }
+
   show_time "opened pipe"
   set ::perlpid [pid $::inst]
 
   # for windows < 10: make sure the main window is still on top
   wm attributes . -topmost
 
-  # do not start event-driven, non-blocking io
-  # until the actual installation starts
   chan configure $::inst -buffering line -blocking 1
 
   # possible input from perl until the menu starts:
   # - question about prior canceled installation
-  # - location (=repository)
+  # - location (actual repository)
   # - menu data, help, version, print-platform
   set ::did_gui 0
   set answer ""
-  while 1 {
+  unset -nocomplain ::loaded
+  while 1 { ; # initial perl output
     set ll [read_line]
-    if {[lindex $ll 0] < 0} break
+    if {[lindex $ll 0] < 0} {
+      break
+    }
     set l [lindex $ll 1]
     # There may be occasion for a dialog
     if {$l eq "mess_yesno"} {
       answer_to_perl
-    } elseif {$l eq "menudata"} {
-      # we do want a menu, so we expect menu data,
-      # which may take a while
+    } elseif [string match "location: ?*" $l] {
+      # this one comes straight from install-tl, rather than
+      # from install-tl-extl.pl
+      # installer about to contact repository, which may
+      # fail and cause an indefinite delay
+      chan configure $::inst -blocking 0
+      chan event $::inst readable read_line_loading
+      if [winfo exists .loading] {
+        .loading configure -text [__ "Trying to load %s.
+
+If this takes too long, press Abort or choose another repository." \
+                                      [string range $l 10 end]]
+        update
+      }
+      break
+    }
+  }
+  # waiting till the repository has been loaded
+  vwait ::loaded
+  unset ::loaded
+  #puts stderr "done loading"
+  # resume reading from back end in blocking mode
+  while 1 {
+    set ll [read_line]
+    if {[lindex $ll 0] < 0} {
+      break
+    }
+    set l [lindex $ll 1]
+    if {$l eq "menudata"} {
+      # so we do want a menu and expect menu data,
+      # parsing which may take a while
       read_menu_data
       show_time "read menu data from perl"
       set ::advanced 0
       set ::alltrees 0
       set answer [run_menu]
-      if [info exists ::env(dbgui)] {puts "dbgui1: menu_ans is $::menu_ans"}
+      #if [info exists ::env(dbgui)] {puts "dbgui1: menu_ans is $::menu_ans"}
       if {$answer eq "advanced"} {
         # this could only happen if $::advanced was 0
         set ::advanced 1
-        if [info exists ::env(dbgui)] {puts "dbgui: Setting advanced to 1"}
+        #if [info exists ::env(dbgui)] {puts "dbgui: Setting advanced to 1"}
         set answer [run_menu]
         if {$answer eq "alltrees"} {
           set ::alltrees 1
-          if [info exists ::env(dbgui)] {puts "dbgui: Setting alltrees to 1"}
+          #if [info exists ::env(dbgui)] {puts "dbgui: Setting alltrees to 1"}
           set answer [run_menu]
         }
       }
@@ -1821,13 +2053,6 @@ proc main_prog {} {
       set ::out_log {}
       set answer "startinst"
       break
-    } elseif [string match "location: ?*" $l] {
-      # this one comes straight from install-tl, rather than
-      # from install-tl-extl.pl
-      if [winfo exists .loading] {
-        .loading configure -text [__ "Loading from %s" [string range $l 10 end]]
-        update
-      }
     } else {
       lappend ::out_log $l
     }
